@@ -7,7 +7,7 @@
 
 import { Model } from '../model';
 import { Repository as BaseRepository, Resource } from '../repository';
-import { InputBox, Git, API, Repository, Remote, RepositoryState, Branch, ForcePushMode, Ref, Submodule, Commit, Change, RepositoryUIState, Status, LogOptions, APIState, CommitOptions, RefType, CredentialsProvider, BranchQuery, PushErrorHandler, PublishEvent, FetchOptions, RemoteSourceProvider, RemoteSourcePublisher, PostCommitCommandsProvider, RefQuery, BranchProtectionProvider, InitOptions, SourceControlHistoryItemDetailsProvider, GitErrorCodes } from './git';
+import { InputBox, Git, API, Repository, Remote, RepositoryState, Branch, ForcePushMode, Ref, Submodule, Commit, Change, RepositoryUIState, Status, LogOptions, APIState, CommitOptions, RefType, CredentialsProvider, BranchQuery, PushErrorHandler, PublishEvent, FetchOptions, RemoteSourceProvider, RemoteSourcePublisher, PostCommitCommandsProvider, RefQuery, BranchProtectionProvider, InitOptions, SourceControlHistoryItemDetailsProvider, GitErrorCodes, CloneOptions, CommitShortStat, DiffChange, Worktree, RepositoryKind, RepositoryAccessDetails } from './git';
 import { Event, SourceControlInputBox, Uri, SourceControl, Disposable, commands, CancellationToken } from 'vscode';
 import { combinedDisposable, filterEvent, mapEvent } from '../util';
 import { toGitUri } from '../uri';
@@ -15,6 +15,7 @@ import { GitExtensionImpl } from './extension';
 import { GitBaseApi } from '../git-base';
 import { PickRemoteSourceOptions } from '../typings/git-base';
 import { OperationKind, OperationResult } from '../operation';
+import { CloneManager } from '../cloneManager';
 
 class ApiInputBox implements InputBox {
 	#inputBox: SourceControlInputBox;
@@ -51,6 +52,7 @@ export class ApiRepositoryState implements RepositoryState {
 	get refs(): Ref[] { console.warn('Deprecated. Use ApiRepository.getRefs() instead.'); return []; }
 	get remotes(): Remote[] { return [...this.#repository.remotes]; }
 	get submodules(): Submodule[] { return [...this.#repository.submodules]; }
+	get worktrees(): Worktree[] { return this.#repository.worktrees; }
 	get rebaseCommit(): Commit | undefined { return this.#repository.rebaseCommit; }
 
 	get mergeChanges(): Change[] { return this.#repository.mergeGroup.resourceStates.map(r => new ApiChange(r)); }
@@ -76,6 +78,7 @@ export class ApiRepository implements Repository {
 
 	readonly rootUri: Uri;
 	readonly inputBox: InputBox;
+	readonly kind: RepositoryKind;
 	readonly state: RepositoryState;
 	readonly ui: RepositoryUIState;
 
@@ -85,6 +88,7 @@ export class ApiRepository implements Repository {
 	constructor(repository: BaseRepository) {
 		this.#repository = repository;
 
+		this.kind = this.#repository.kind;
 		this.rootUri = Uri.file(this.#repository.root);
 		this.inputBox = new ApiInputBox(this.#repository.inputBox);
 		this.state = new ApiRepositoryState(this.#repository);
@@ -96,8 +100,11 @@ export class ApiRepository implements Repository {
 			filterEvent(this.#repository.onDidRunOperation, e => e.operation.kind === OperationKind.Checkout || e.operation.kind === OperationKind.CheckoutTracking), () => null);
 	}
 
-	apply(patch: string, reverse?: boolean): Promise<void> {
-		return this.#repository.apply(patch, reverse);
+	apply(patch: string, reverse?: boolean): Promise<void>;
+	apply(patch: string, options?: { allowEmpty?: boolean; reverse?: boolean; threeWay?: boolean }): Promise<void>;
+	apply(patch: string, reverseOrOptions?: boolean | { allowEmpty?: boolean; reverse?: boolean; threeWay?: boolean }): Promise<void> {
+		const options = typeof reverseOrOptions === 'boolean' ? { reverse: reverseOrOptions } : reverseOrOptions;
+		return this.#repository.apply(patch, options);
 	}
 
 	getConfigs(): Promise<{ key: string; value: string }[]> {
@@ -162,6 +169,10 @@ export class ApiRepository implements Repository {
 		return this.#repository.diffWithHEAD(path);
 	}
 
+	diffWithHEADShortStats(path?: string): Promise<CommitShortStat> {
+		return this.#repository.diffWithHEADShortStats(path);
+	}
+
 	diffWith(ref: string): Promise<Change[]>;
 	diffWith(ref: string, path: string): Promise<string>;
 	diffWith(ref: string, path?: string): Promise<string | Change[]> {
@@ -174,6 +185,10 @@ export class ApiRepository implements Repository {
 		return this.#repository.diffIndexWithHEAD(path);
 	}
 
+	diffIndexWithHEADShortStats(path?: string): Promise<CommitShortStat> {
+		return this.#repository.diffIndexWithHEADShortStats(path);
+	}
+
 	diffIndexWith(ref: string): Promise<Change[]>;
 	diffIndexWith(ref: string, path: string): Promise<string>;
 	diffIndexWith(ref: string, path?: string): Promise<string | Change[]> {
@@ -190,6 +205,14 @@ export class ApiRepository implements Repository {
 		return this.#repository.diffBetween(ref1, ref2, path);
 	}
 
+	diffBetweenPatch(ref1: string, ref2: string, path?: string): Promise<string> {
+		return this.#repository.diffBetweenPatch(ref1, ref2, path);
+	}
+
+	diffBetweenWithStats(ref1: string, ref2: string, path?: string): Promise<DiffChange[]> {
+		return this.#repository.diffBetweenWithStats(ref1, ref2, path);
+	}
+
 	hashObject(data: string): Promise<string> {
 		return this.#repository.hashObject(data);
 	}
@@ -298,6 +321,10 @@ export class ApiRepository implements Repository {
 		return this.#repository.mergeAbort();
 	}
 
+	createStash(options?: { message?: string; includeUntracked?: boolean; staged?: boolean }): Promise<void> {
+		return this.#repository.createStash(options?.message, options?.includeUntracked, options?.staged);
+	}
+
 	applyStash(index?: number): Promise<void> {
 		return this.#repository.applyStash(index);
 	}
@@ -309,6 +336,18 @@ export class ApiRepository implements Repository {
 	dropStash(index?: number): Promise<void> {
 		return this.#repository.dropStash(index);
 	}
+
+	createWorktree(options?: { path?: string; commitish?: string; branch?: string }): Promise<string> {
+		return this.#repository.createWorktree(options);
+	}
+
+	deleteWorktree(path: string, options?: { force?: boolean }): Promise<void> {
+		return this.#repository.deleteWorktree(path, options);
+	}
+
+	migrateChanges(sourceRepositoryPath: string, options?: { confirmation?: boolean; deleteFromSource?: boolean; untracked?: boolean }): Promise<void> {
+		return this.#repository.migrateChanges(sourceRepositoryPath, options);
+	}
 }
 
 export class ApiGit implements Git {
@@ -331,10 +370,12 @@ export class ApiGit implements Git {
 
 export class ApiImpl implements API {
 	#model: Model;
+	#cloneManager: CloneManager;
 	readonly git: ApiGit;
 
-	constructor(model: Model) {
-		this.#model = model;
+	constructor(privates: { model: Model; cloneManager: CloneManager }) {
+		this.#model = privates.model;
+		this.#cloneManager = privates.cloneManager;
 		this.git = new ApiGit(this.#model);
 	}
 
@@ -362,6 +403,10 @@ export class ApiImpl implements API {
 		return this.#model.repositories.map(r => new ApiRepository(r));
 	}
 
+	get recentRepositories(): Iterable<RepositoryAccessDetails> {
+		return this.#model.repositoryCache.recentRepositories;
+	}
+
 	toGitUri(uri: Uri, ref: string): Uri {
 		return toGitUri(uri, ref);
 	}
@@ -392,6 +437,11 @@ export class ApiImpl implements API {
 		}
 	}
 
+	async getRepositoryWorkspace(uri: Uri): Promise<Uri[] | null> {
+		const workspaces = this.#model.repositoryCache.get(uri.toString());
+		return workspaces ? workspaces.map(r => Uri.file(r.workspacePath)) : null;
+	}
+
 	async init(root: Uri, options?: InitOptions): Promise<Repository | null> {
 		const path = root.fsPath;
 		await this.#model.git.init(path, options);
@@ -399,6 +449,12 @@ export class ApiImpl implements API {
 		return this.getRepository(root) || null;
 	}
 
+	async clone(uri: Uri, options?: CloneOptions): Promise<Uri | null> {
+		const parentPath = options?.parentPath?.fsPath;
+		const result = await this.#cloneManager.clone(uri.toString(), { parentPath, recursive: options?.recursive, ref: options?.ref, postCloneAction: options?.postCloneAction });
+		return result ? Uri.file(result) : null;
+	}
+
 	async openRepository(root: Uri): Promise<Repository | null> {
 		if (root.scheme !== 'file') {
 			return null;
@@ -511,6 +567,7 @@ export function registerAPICommands(extension: GitExtensionImpl): Disposable {
 			refs: state.refs.map(ref),
 			remotes: state.remotes,
 			submodules: state.submodules,
+			worktrees: state.worktrees,
 			rebaseCommit: state.rebaseCommit,
 			mergeChanges: state.mergeChanges.map(change),
 			indexChanges: state.indexChanges.map(change),
