@@ -3,11 +3,12 @@
  *  Licensed under the MIT License. See License.txt in the project root for license information.
  *--------------------------------------------------------------------------------------------*/
 
-import { Event, Disposable, EventEmitter, SourceControlHistoryItemRef, l10n, workspace, Uri, DiagnosticSeverity, env } from 'vscode';
-import { dirname, sep, relative } from 'path';
+import { Event, Disposable, EventEmitter, SourceControlHistoryItemRef, l10n, workspace, Uri, DiagnosticSeverity, env, SourceControlHistoryItem } from 'vscode';
+import { dirname, normalize, sep, relative } from 'path';
 import { Readable } from 'stream';
 import { promises as fs, createReadStream } from 'fs';
 import byline from 'byline';
+import { Stash } from './git';
 
 export const isMacintosh = process.platform === 'darwin';
 export const isWindows = process.platform === 'win32';
@@ -15,6 +16,10 @@ export const isRemote = env.remoteName !== undefined;
 export const isLinux = process.platform === 'linux';
 export const isLinuxSnap = isLinux && !!process.env['SNAP'] && !!process.env['SNAP_REVISION'];
 
+export type Mutable<T> = {
+	-readonly [P in keyof T]: T[P]
+};
+
 export function log(...args: any[]): void {
 	console.log.apply(console, ['git:', ...args]);
 }
@@ -38,10 +43,6 @@ export function combinedDisposable(disposables: IDisposable[]): IDisposable {
 
 export const EmptyDisposable = toDisposable(() => null);
 
-export function fireEvent<T>(event: Event<T>): Event<T> {
-	return (listener: (e: T) => any, thisArgs?: any, disposables?: Disposable[]) => event(_ => (listener as any).call(thisArgs), null, disposables);
-}
-
 export function mapEvent<I, O>(event: Event<I>, map: (i: I) => O): Event<O> {
 	return (listener: (e: O) => any, thisArgs?: any, disposables?: Disposable[]) => event(i => listener.call(thisArgs, map(i)), null, disposables);
 }
@@ -110,7 +111,8 @@ export function once(fn: (...args: any[]) => any): (...args: any[]) => any {
 
 export function assign<T>(destination: T, ...sources: any[]): T {
 	for (const source of sources) {
-		Object.keys(source).forEach(key => (destination as any)[key] = source[key]);
+		Object.keys(source).forEach(key =>
+			(destination as Record<string, unknown>)[key] = source[key]);
 	}
 
 	return destination;
@@ -139,6 +141,9 @@ export function groupBy<T>(arr: T[], fn: (el: T) => string): { [key: string]: T[
 	}, Object.create(null));
 }
 
+export function coalesce<T>(array: ReadonlyArray<T | undefined>): T[] {
+	return array.filter((e): e is T => !!e);
+}
 
 export async function mkdirp(path: string, mode?: number): Promise<boolean> {
 	const mkdir = async () => {
@@ -236,7 +241,7 @@ export function readBytes(stream: Readable, bytes: number): Promise<Buffer> {
 			bytesRead += bytesToRead;
 
 			if (bytesRead === bytes) {
-				(stream as any).destroy(); // Will trigger the close event eventually
+				stream.destroy(); // Will trigger the close event eventually
 			}
 		});
 
@@ -295,14 +300,26 @@ export function truncate(value: string, maxLength = 20, ellipsis = true): string
 	return value.length <= maxLength ? value : `${value.substring(0, maxLength)}${ellipsis ? '\u2026' : ''}`;
 }
 
+export function subject(value: string): string {
+	const index = value.indexOf('\n');
+	return index === -1 ? value : truncate(value, index, false);
+}
+
 function normalizePath(path: string): string {
 	// Windows & Mac are currently being handled
 	// as case insensitive file systems in VS Code.
 	if (isWindows || isMacintosh) {
-		return path.toLowerCase();
+		path = path.toLowerCase();
 	}
 
-	return path;
+	// Trailing separator
+	if (/[/\\]$/.test(path)) {
+		// Remove trailing separator
+		path = path.substring(0, path.length - 1);
+	}
+
+	// Normalize the path
+	return normalize(path);
 }
 
 export function isDescendant(parent: string, descendant: string): boolean {
@@ -310,11 +327,16 @@ export function isDescendant(parent: string, descendant: string): boolean {
 		return true;
 	}
 
+	// Normalize the paths
+	parent = normalizePath(parent);
+	descendant = normalizePath(descendant);
+
+	// Ensure parent ends with separator
 	if (parent.charAt(parent.length - 1) !== sep) {
 		parent += sep;
 	}
 
-	return normalizePath(descendant).startsWith(normalizePath(parent));
+	return descendant.startsWith(parent);
 }
 
 export function pathEquals(a: string, b: string): boolean {
@@ -779,6 +801,12 @@ export function getCommitShortHash(scope: Uri, hash: string): string {
 	return hash.substring(0, shortHashLength);
 }
 
+export function getHistoryItemDisplayName(historyItem: SourceControlHistoryItem): string {
+	return historyItem.references?.length
+		? historyItem.references[0].name
+		: historyItem.displayId ?? historyItem.id;
+}
+
 export type DiagnosticSeverityConfig = 'error' | 'warning' | 'information' | 'hint' | 'none';
 
 export function toDiagnosticSeverity(value: DiagnosticSeverityConfig): DiagnosticSeverity {
@@ -790,3 +818,59 @@ export function toDiagnosticSeverity(value: DiagnosticSeverityConfig): Diagnosti
 				? DiagnosticSeverity.Information
 				: DiagnosticSeverity.Hint;
 }
+
+export function extractFilePathFromArgs(argv: string[], startIndex: number): string {
+	// Argument doesn't start with a quote
+	const firstArg = argv[startIndex];
+	if (!firstArg.match(/^["']/)) {
+		return firstArg.replace(/^["']+|["':]+$/g, '');
+	}
+
+	// If it starts with a quote, we need to find the matching closing
+	// quote which might be in a later argument if the path contains
+	// spaces
+	const quote = firstArg[0];
+
+	// If the first argument ends with the same quote, it's complete
+	if (firstArg.endsWith(quote) && firstArg.length > 1) {
+		return firstArg.slice(1, -1);
+	}
+
+	// Concatenate arguments until we find the closing quote
+	let path = firstArg;
+	for (let i = startIndex + 1; i < argv.length; i++) {
+		path = `${path} ${argv[i]}`;
+		if (argv[i].endsWith(quote)) {
+			// Found the matching quote
+			return path.slice(1, -1);
+		}
+	}
+
+	// If no closing quote was found, remove
+	// leading quote and return the path as-is
+	return path.slice(1);
+}
+
+export function getStashDescription(stash: Stash): string | undefined {
+	if (!stash.commitDate && !stash.branchName) {
+		return undefined;
+	}
+
+	const descriptionSegments: string[] = [];
+	if (stash.commitDate) {
+		descriptionSegments.push(fromNow(stash.commitDate));
+	}
+	if (stash.branchName) {
+		descriptionSegments.push(stash.branchName);
+	}
+
+	return descriptionSegments.join(' \u2022 ');
+}
+
+export function isCopilotWorktree(path: string): boolean {
+	const lastSepIndex = path.lastIndexOf(sep);
+
+	return lastSepIndex !== -1
+		? path.substring(lastSepIndex + 1).startsWith('copilot-worktree-')
+		: path.startsWith('copilot-worktree-');
+}
