@@ -7,9 +7,12 @@ import { IKeyboardEvent } from '../../base/browser/keyboardEvent.js';
 import { IMouseEvent, IMouseWheelEvent } from '../../base/browser/mouseEvent.js';
 import { IBoundarySashes } from '../../base/browser/ui/sash/sash.js';
 import { Event } from '../../base/common/event.js';
-import { IEditorConstructionOptions } from './config/editorConfiguration.js';
+import { MenuId } from '../../platform/actions/common/actions.js';
+import { IContextKeyService } from '../../platform/contextkey/common/contextkey.js';
+import { ServicesAccessor } from '../../platform/instantiation/common/instantiation.js';
 import { ConfigurationChangedEvent, EditorLayoutInfo, EditorOption, FindComputedEditorOptionValueById, IComputedEditorOptions, IDiffEditorOptions, IEditorOptions, OverviewRulerPosition } from '../common/config/editorOptions.js';
 import { IDimension } from '../common/core/2d/dimension.js';
+import { TextEdit } from '../common/core/edits/textEdit.js';
 import { IPosition, Position } from '../common/core/position.js';
 import { IRange, Range } from '../common/core/range.js';
 import { Selection } from '../common/core/selection.js';
@@ -17,14 +20,14 @@ import { IWordAtPosition } from '../common/core/wordHelper.js';
 import { ICursorPositionChangedEvent, ICursorSelectionChangedEvent } from '../common/cursorEvents.js';
 import { IDiffComputationResult, ILineChange } from '../common/diff/legacyLinesDiffComputer.js';
 import * as editorCommon from '../common/editorCommon.js';
-import { GlyphMarginLane, ICursorStateComputer, IIdentifiedSingleEditOperation, IModelDecoration, IModelDeltaDecoration, ITextModel, PositionAffinity } from '../common/model.js';
+import { GlyphMarginLane, ICursorStateComputer, IIdentifiedSingleEditOperation, IModelDecoration, IModelDecorationsChangeAccessor, IModelDeltaDecoration, ITextModel, PositionAffinity } from '../common/model.js';
 import { InjectedText } from '../common/modelLineProjectionData.js';
-import { IModelContentChangedEvent, IModelDecorationsChangedEvent, IModelLanguageChangedEvent, IModelLanguageConfigurationChangedEvent, IModelOptionsChangedEvent, IModelTokensChangedEvent, ModelLineHeightChangedEvent } from '../common/textModelEvents.js';
+import { TextModelEditSource } from '../common/textModelEditSource.js';
+import { IModelContentChangedEvent, IModelDecorationsChangedEvent, IModelLanguageChangedEvent, IModelLanguageConfigurationChangedEvent, IModelOptionsChangedEvent, IModelTokensChangedEvent, ModelFontChangedEvent, ModelLineHeightChangedEvent } from '../common/textModelEvents.js';
 import { IEditorWhitespace, IViewModel } from '../common/viewModel.js';
 import { OverviewRulerZone } from '../common/viewModel/overviewZoneManager.js';
-import { MenuId } from '../../platform/actions/common/actions.js';
-import { IContextKeyService } from '../../platform/contextkey/common/contextkey.js';
-import { ServicesAccessor } from '../../platform/instantiation/common/instantiation.js';
+import { IEditorConstructionOptions } from './config/editorConfiguration.js';
+import { IClipboardCopyEvent, IClipboardPasteEvent } from './controller/editContext/clipboardUtils.js';
 
 /**
  * A view zone is a full horizontal rectangle that 'pushes' text down.
@@ -176,6 +179,13 @@ export interface IContentWidget {
 	 * Render this content widget in a location where it could overflow the editor's view dom node.
 	 */
 	allowEditorOverflow?: boolean;
+
+	/**
+	 * If true, this widget doesn't have a visual representation.
+	 * The element will have display set to 'none'.
+	*/
+	useDisplayNone?: boolean;
+
 	/**
 	 * Call preventDefault() on mousedown events that target the content widget.
 	 */
@@ -257,6 +267,8 @@ export interface IOverlayWidgetPositionCoordinates {
 	left: number;
 }
 
+
+
 /**
  * A position for rendering overlay widgets.
  */
@@ -270,7 +282,7 @@ export interface IOverlayWidgetPosition {
 	 * When set, stacks with other overlay widgets with the same preference,
 	 * in an order determined by the ordinal value.
 	 */
-	stackOridinal?: number;
+	stackOrdinal?: number;
 }
 /**
  * An overlay widgets renders on top of the text.
@@ -279,7 +291,7 @@ export interface IOverlayWidget {
 	/**
 	 * Event fired when the widget layout changes.
 	 */
-	onDidLayout?: Event<void>;
+	readonly onDidLayout?: Event<void>;
 	/**
 	 * Render this overlay widget in a location where it could overflow the editor's view dom node.
 	 */
@@ -716,6 +728,24 @@ export interface ICodeEditor extends editorCommon.IEditor {
 	 * @event
 	 */
 	readonly onDidPaste: Event<IPasteEvent>;
+	/**
+	 * An event emitted before clipboard copy operation starts.
+	 * @internal
+	 * @event
+	 */
+	readonly onWillCopy: Event<IClipboardCopyEvent>;
+	/**
+	 * An event emitted before clipboard cut operation starts.
+	 * @internal
+	 * @event
+	 */
+	readonly onWillCut: Event<IClipboardCopyEvent>;
+	/**
+	 * An event emitted before clipboard paste operation starts.
+	 * @internal
+	 * @event
+	 */
+	readonly onWillPaste: Event<IClipboardPasteEvent>;
 	/**
 	 * An event emitted on a "mouseup".
 	 * @event
@@ -823,6 +853,8 @@ export interface ICodeEditor extends editorCommon.IEditor {
 	*/
 	readonly onEndUpdate: Event<void>;
 
+	readonly onDidChangeViewZones: Event<void>;
+
 	/**
 	 * Saves current view state of the editor in a serializable object.
 	 */
@@ -896,7 +928,14 @@ export interface ICodeEditor extends editorCommon.IEditor {
 	 * @internal
 	 * @event
 	 */
-	onDidChangeLineHeight: Event<ModelLineHeightChangedEvent>;
+	readonly onDidChangeLineHeight: Event<ModelLineHeightChangedEvent>;
+
+	/**
+	 * An event emitted when the font of the editor has changed.
+	 * @internal
+	 * @event
+	 */
+	readonly onDidChangeFont: Event<ModelFontChangedEvent>;
 
 	/**
 	 * Get value of the current model attached to this editor.
@@ -988,6 +1027,13 @@ export interface ICodeEditor extends editorCommon.IEditor {
 	 * @param endCursorState Cursor state after the edits were applied.
 	 */
 	executeEdits(source: string | null | undefined, edits: IIdentifiedSingleEditOperation[], endCursorState?: ICursorStateComputer | Selection[]): boolean;
+	/** @internal */
+	executeEdits(source: TextModelEditSource | undefined, edits: IIdentifiedSingleEditOperation[], endCursorState?: ICursorStateComputer | Selection[]): boolean;
+
+	/**
+	 * @internal
+	*/
+	edit(edit: TextEdit, reason: TextModelEditSource): void;
 
 	/**
 	 * Execute multiple (concomitant) commands on the editor.
@@ -996,6 +1042,11 @@ export interface ICodeEditor extends editorCommon.IEditor {
 	 */
 	executeCommands(source: string | null | undefined, commands: (editorCommon.ICommand | null)[]): void;
 
+	/**
+	 * Scroll vertically or horizontally as necessary and reveal the current cursors.
+	 */
+	revealAllCursors(revealHorizontal: boolean, minimalReveal?: boolean): void;
+
 	/**
 	 * @internal
 	 */
@@ -1011,6 +1062,12 @@ export interface ICodeEditor extends editorCommon.IEditor {
 	 */
 	getDecorationsInRange(range: Range): IModelDecoration[] | null;
 
+	/**
+	 * Get the font size at a given position
+	 * @param position the position for which to fetch the font size
+	 */
+	getFontSizeAtPosition(position: IPosition): string | null;
+
 	/**
 	 * All decorations added through this call will get the ownerId of this editor.
 	 * @deprecated Use `createDecorationsCollection`
@@ -1076,7 +1133,7 @@ export interface ICodeEditor extends editorCommon.IEditor {
 	getTopForPosition(lineNumber: number, column: number): number;
 
 	/**
-	 * Get the line height for the line number.
+	 * Get the line height for a model position.
 	 */
 	getLineHeightForPosition(position: IPosition): number;
 
@@ -1101,7 +1158,7 @@ export interface ICodeEditor extends editorCommon.IEditor {
 	/**
 	 * @internal
 	 */
-	getTelemetryData(): { [key: string]: any } | undefined;
+	getTelemetryData(): object | undefined;
 
 	/**
 	 * Returns the editor's container dom node
@@ -1167,11 +1224,24 @@ export interface ICodeEditor extends editorCommon.IEditor {
 	 */
 	getOffsetForColumn(lineNumber: number, column: number): number;
 
+	getWidthOfLine(lineNumber: number): number;
+
+	/**
+	 * Reset cached line widths. Call this when the editor becomes visible after being hidden.
+	 * @internal
+	 */
+	resetLineWidthCaches(): void;
+
 	/**
 	 * Force an editor render now.
 	 */
 	render(forceRedraw?: boolean): void;
 
+	/**
+	 * Render the editor at the next animation frame.
+	 */
+	renderAsync(forceRedraw?: boolean): void;
+
 	/**
 	 * Get the hit test target at coordinates `clientX` and `clientY`.
 	 * The coordinates are relative to the top-left of the viewport.
@@ -1261,6 +1331,15 @@ export interface IActiveCodeEditor extends ICodeEditor {
 	 * Warning: the results of this method are inaccurate for positions that are outside the current editor viewport.
 	 */
 	getScrolledVisiblePosition(position: IPosition): { top: number; left: number; height: number };
+
+	/**
+	 * Change the decorations. All decorations added through this changeAccessor
+	 * will get the ownerId of the editor (meaning they will not show up in other
+	 * editors).
+	 * @see {@link ITextModel.changeDecorations}
+	 * @internal
+	 */
+	changeDecorations<T>(callback: (changeAccessor: IModelDecorationsChangeAccessor) => T): T;
 }
 
 /**
@@ -1440,10 +1519,20 @@ export function getCodeEditor(thing: unknown): ICodeEditor | null {
 /**
  *@internal
  */
-export function getIEditor(thing: any): editorCommon.IEditor | null {
+export function getIEditor(thing: unknown): editorCommon.IEditor | null {
 	if (isCodeEditor(thing) || isDiffEditor(thing)) {
 		return thing;
 	}
 
 	return null;
 }
+
+/**
+ *@internal
+ */
+export function isIOverlayWidgetPositionCoordinates(thing: unknown): thing is IOverlayWidgetPositionCoordinates {
+	return !!thing
+		&& typeof thing === 'object'
+		&& typeof (<IOverlayWidgetPositionCoordinates>thing).top === 'number'
+		&& typeof (<IOverlayWidgetPositionCoordinates>thing).left === 'number';
+}
