@@ -18,6 +18,7 @@ import { bindContextKey } from '../../../../platform/observable/common/platformO
 import { IEditorProgressService } from '../../../../platform/progress/common/progress.js';
 import { IDiffEditorOptions } from '../../../common/config/editorOptions.js';
 import { IDimension } from '../../../common/core/2d/dimension.js';
+import { LineRange } from '../../../common/core/ranges/lineRange.js';
 import { Position } from '../../../common/core/position.js';
 import { Range } from '../../../common/core/range.js';
 import { CursorChangeReason, ICursorPositionChangedEvent } from '../../../common/cursorEvents.js';
@@ -91,7 +92,7 @@ export class DiffEditorWidget extends DelegatingEditor implements IDiffEditor {
 		codeEditorWidgetOptions: IDiffCodeEditorWidgetOptions,
 		@IContextKeyService private readonly _parentContextKeyService: IContextKeyService,
 		@IInstantiationService private readonly _parentInstantiationService: IInstantiationService,
-		@ICodeEditorService codeEditorService: ICodeEditorService,
+		@ICodeEditorService private readonly _codeEditorService: ICodeEditorService,
 		@IAccessibilitySignalService private readonly _accessibilitySignalService: IAccessibilitySignalService,
 		@IEditorProgressService private readonly _editorProgressService: IEditorProgressService,
 	) {
@@ -179,7 +180,7 @@ export class DiffEditorWidget extends DelegatingEditor implements IDiffEditor {
 		});
 		this._diffValue = this._diffModel.map((m, r) => m?.diff.read(r));
 		this.onDidUpdateDiff = Event.fromObservableLight(this._diffValue);
-		codeEditorService.willCreateDiffEditor();
+		this._codeEditorService.willCreateDiffEditor();
 
 		this._contextKeyService.createKey('isInDiffEditor', true);
 
@@ -293,7 +294,7 @@ export class DiffEditorWidget extends DelegatingEditor implements IDiffEditor {
 				this._diffModel,
 				this._options,
 				this,
-				() => isUpdatingViewZones || unchangedRangesFeature.get().isUpdatingHiddenAreas,
+				() => isUpdatingViewZones || unchangedRangesFeature.read(undefined).isUpdatingHiddenAreas,
 				origViewZoneIdsToIgnore,
 				modViewZoneIdsToIgnore
 			)
@@ -343,7 +344,10 @@ export class DiffEditorWidget extends DelegatingEditor implements IDiffEditor {
 
 		this._createDiffEditorContributions();
 
-		codeEditorService.addDiffEditor(this);
+		this._codeEditorService.addDiffEditor(this);
+		this._register(toDisposable(() => {
+			this._codeEditorService.removeDiffEditor(this);
+		}));
 
 		this._gutter = derivedDisposable(this, reader => {
 			return this._options.shouldRenderGutterMenu.read(reader)
@@ -469,10 +473,11 @@ export class DiffEditorWidget extends DelegatingEditor implements IDiffEditor {
 
 	public override restoreViewState(s: IDiffEditorViewState): void {
 		if (s && s.original && s.modified) {
-			const diffEditorState = s as IDiffEditorViewState;
+			const diffEditorState = s;
 			this._editors.original.restoreViewState(diffEditorState.original);
 			this._editors.modified.restoreViewState(diffEditorState.modified);
 			if (diffEditorState.modelState) {
+				// eslint-disable-next-line local/code-no-any-casts, @typescript-eslint/no-explicit-any
 				this._diffModel.get()?.restoreSerializedState(diffEditorState.modelState as any);
 			}
 		}
@@ -573,12 +578,14 @@ export class DiffEditorWidget extends DelegatingEditor implements IDiffEditor {
 		const model = this._diffModel.get();
 		if (!model || !model.isDiffUpToDate.get()) { return; }
 
+		this._editors.modified.pushUndoStop();
 		this._editors.modified.executeEdits('diffEditor', [
 			{
 				range: diff.modified.toExclusiveRange(),
 				text: model.model.original.getValueInRange(diff.original.toExclusiveRange())
 			}
 		]);
+		this._editors.modified.pushUndoStop();
 	}
 
 	revertRangeMappings(diffs: RangeMapping[]): void {
@@ -590,9 +597,39 @@ export class DiffEditorWidget extends DelegatingEditor implements IDiffEditor {
 			text: model.model.original.getValueInRange(c.originalRange)
 		}));
 
+		this._editors.modified.pushUndoStop();
 		this._editors.modified.executeEdits('diffEditor', changes);
+		this._editors.modified.pushUndoStop();
 	}
 
+	revertFocusedRangeMappings() {
+		const model = this._diffModel.get();
+		if (!model || !model.isDiffUpToDate.get()) { return; }
+
+		const diffs = this._diffModel.get()?.diff.get()?.mappings;
+		if (!diffs || diffs.length === 0) { return; }
+
+		const modifiedEditor = this._editors.modified;
+		if (!modifiedEditor.hasTextFocus()) { return; }
+
+		const curLineNumber = modifiedEditor.getPosition()!.lineNumber;
+		const selection = modifiedEditor.getSelection();
+		const selectedRange = LineRange.fromRange(selection || new Range(curLineNumber, 0, curLineNumber, 0));
+		const diffsToRevert = diffs.filter(d => {
+			return d.lineRangeMapping.modified.intersect(selectedRange);
+		});
+
+		modifiedEditor.pushUndoStop();
+		modifiedEditor.executeEdits('diffEditor', diffsToRevert.map(d => (
+			{
+				range: d.lineRangeMapping.modified.toExclusiveRange(),
+				text: model.model.original.getValueInRange(d.lineRangeMapping.original.toExclusiveRange())
+			}
+		)));
+		modifiedEditor.pushUndoStop();
+	}
+
+
 	private _goTo(diff: DiffMapping): void {
 		this._editors.modified.setPosition(new Position(diff.lineRangeMapping.modified.startLineNumber, 1));
 		this._editors.modified.revealRangeInCenter(diff.lineRangeMapping.modified.toExclusiveRange());
@@ -605,10 +642,14 @@ export class DiffEditorWidget extends DelegatingEditor implements IDiffEditor {
 		}
 
 		const curLineNumber = this._editors.modified.getPosition()!.lineNumber;
-
 		let diff: DiffMapping | undefined;
 		if (target === 'next') {
-			diff = diffs.find(d => d.lineRangeMapping.modified.startLineNumber > curLineNumber) ?? diffs[0];
+			const modifiedLineCount = this._editors.modified.getModel()!.getLineCount();
+			if (modifiedLineCount === curLineNumber) {
+				diff = diffs[0];
+			} else {
+				diff = diffs.find(d => d.lineRangeMapping.modified.startLineNumber > curLineNumber) ?? diffs[0];
+			}
 		} else {
 			diff = findLast(diffs, d => d.lineRangeMapping.modified.startLineNumber < curLineNumber) ?? diffs[diffs.length - 1];
 		}
