@@ -3,7 +3,6 @@
  *  Licensed under the MIT License. See License.txt in the project root for license information.
  *--------------------------------------------------------------------------------------------*/
 
-import { addDisposableListener } from '../../../../base/browser/dom.js';
 import { IAction } from '../../../../base/common/actions.js';
 import { coalesce } from '../../../../base/common/arrays.js';
 import { CancelablePromise, createCancelablePromise, DeferredPromise, raceCancellation } from '../../../../base/common/async.js';
@@ -13,7 +12,6 @@ import { isCancellationError } from '../../../../base/common/errors.js';
 import { HierarchicalKind } from '../../../../base/common/hierarchicalKind.js';
 import { Disposable, DisposableStore } from '../../../../base/common/lifecycle.js';
 import { Mimes } from '../../../../base/common/mime.js';
-import * as platform from '../../../../base/common/platform.js';
 import { upcast } from '../../../../base/common/types.js';
 import { generateUuid } from '../../../../base/common/uuid.js';
 import { localize } from '../../../../nls.js';
@@ -22,14 +20,13 @@ import { ICommandService } from '../../../../platform/commands/common/commands.j
 import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
 import { RawContextKey } from '../../../../platform/contextkey/common/contextkey.js';
 import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
+import { ILogService } from '../../../../platform/log/common/log.js';
 import { IProgressService, ProgressLocation } from '../../../../platform/progress/common/progress.js';
 import { IQuickInputService, IQuickPickItem, IQuickPickSeparator } from '../../../../platform/quickinput/common/quickInput.js';
-import { ClipboardEventUtils } from '../../../browser/controller/editContext/clipboardUtils.js';
-import { toExternalVSDataTransfer, toVSDataTransfer } from '../../../browser/dataTransfer.js';
+import { IClipboardCopyEvent, IClipboardPasteEvent, IWritableClipboardData } from '../../../browser/controller/editContext/clipboardUtils.js';
 import { ICodeEditor, PastePayload } from '../../../browser/editorBrowser.js';
 import { IBulkEditService } from '../../../browser/services/bulkEditService.js';
 import { EditorOption } from '../../../common/config/editorOptions.js';
-import { IRange, Range } from '../../../common/core/range.js';
 import { Selection } from '../../../common/core/selection.js';
 import { Handler, IEditorContribution } from '../../../common/editorCommon.js';
 import { DocumentPasteContext, DocumentPasteEdit, DocumentPasteEditProvider, DocumentPasteTriggerKind } from '../../../common/languages.js';
@@ -116,6 +113,7 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 	constructor(
 		editor: ICodeEditor,
 		@IInstantiationService instantiationService: IInstantiationService,
+		@ILogService private readonly _logService: ILogService,
 		@IBulkEditService private readonly _bulkEditService: IBulkEditService,
 		@IClipboardService private readonly _clipboardService: IClipboardService,
 		@ICommandService private readonly _commandService: ICommandService,
@@ -128,10 +126,9 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 
 		this._editor = editor;
 
-		const container = editor.getContainerDomNode();
-		this._register(addDisposableListener(container, 'copy', e => this.handleCopy(e)));
-		this._register(addDisposableListener(container, 'cut', e => this.handleCopy(e)));
-		this._register(addDisposableListener(container, 'paste', e => this.handlePaste(e), true));
+		this._register(editor.onWillCopy(e => this.handleCopy(e)));
+		this._register(editor.onWillCut(e => this.handleCopy(e)));
+		this._register(editor.onWillPaste(e => this.handlePaste(e)));
 
 		this._pasteProgressManager = this._register(new InlineProgressManager('pasteIntoEditor', editor, instantiationService));
 
@@ -146,8 +143,10 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 	}
 
 	public async pasteAs(preferred?: PastePreference) {
+		this._logService.trace('CopyPasteController.pasteAs');
 		this._editor.focus();
 		try {
+			this._logService.trace('Before calling editor.action.clipboardPasteAction');
 			this._pasteAsActionContext = { preferred };
 			await this._commandService.executeCommand('editor.action.clipboardPasteAction');
 		} finally {
@@ -167,7 +166,8 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 		await this._currentPasteOperation;
 	}
 
-	private handleCopy(e: ClipboardEvent) {
+	private handleCopy(e: IClipboardCopyEvent) {
+		this._logService.trace('CopyPasteController#handleCopy');
 		if (!this._editor.hasTextFocus()) {
 			return;
 		}
@@ -177,34 +177,20 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 		// This means the resources clipboard is not properly updated when copying from the editor.
 		this._clipboardService.clearInternalState?.();
 
-		if (!e.clipboardData || !this.isPasteAsEnabled()) {
+		if (!this.isPasteAsEnabled()) {
 			return;
 		}
 
 		const model = this._editor.getModel();
+		const viewModel = this._editor._getViewModel();
 		const selections = this._editor.getSelections();
-		if (!model || !selections?.length) {
+		if (!model || !viewModel || !selections?.length) {
 			return;
 		}
 
-		const enableEmptySelectionClipboard = this._editor.getOption(EditorOption.emptySelectionClipboard);
-
-		let ranges: readonly IRange[] = selections;
-		const wasFromEmptySelection = selections.length === 1 && selections[0].isEmpty();
-		if (wasFromEmptySelection) {
-			if (!enableEmptySelectionClipboard) {
-				return;
-			}
-
-			ranges = [new Range(ranges[0].startLineNumber, 1, ranges[0].startLineNumber, 1 + model.getLineLength(ranges[0].startLineNumber))];
-		}
-
-		const toCopy = this._editor._getViewModel()?.getPlainTextToCopy(selections, enableEmptySelectionClipboard, platform.isWindows);
-		const multicursorText = Array.isArray(toCopy) ? toCopy : null;
-
 		const defaultPastePayload = {
-			multicursorText,
-			pasteOnNewLine: wasFromEmptySelection,
+			multicursorText: e.dataToCopy.multicursorText ?? null,
+			pasteOnNewLine: e.dataToCopy.isFromEmptySelection,
 			mode: null
 		};
 
@@ -216,7 +202,7 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 			return;
 		}
 
-		const dataTransfer = toVSDataTransfer(e.clipboardData);
+		const dataTransfer = new VSDataTransfer();
 		const providerCopyMimeTypes = providers.flatMap(x => x.copyMimeTypes ?? []);
 
 		// Save off a handle pointing to data that VS Code maintains.
@@ -231,7 +217,7 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 			return {
 				providerMimeTypes: provider.copyMimeTypes,
 				operation: createCancelablePromise(token =>
-					provider.prepareDocumentPaste!(model, ranges, dataTransfer, token)
+					provider.prepareDocumentPaste!(model, e.dataToCopy.sourceRanges, dataTransfer, token)
 						.catch(err => {
 							console.error(err);
 							return undefined;
@@ -243,11 +229,19 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 		CopyPasteController._currentCopyOperation = { handle, operations };
 	}
 
-	private async handlePaste(e: ClipboardEvent) {
-		if (!e.clipboardData || !this._editor.hasTextFocus()) {
+	private async handlePaste(e: IClipboardPasteEvent) {
+		this._logService.trace('CopyPasteController#handlePaste for id : ', e.metadata?.id);
+
+		if (!this._editor.hasTextFocus()) {
 			return;
 		}
 
+		const dataTransfer = e.toExternalVSDataTransfer();
+		if (!dataTransfer) {
+			return;
+		}
+		dataTransfer.delete(vscodeClipboardMime);
+
 		MessageController.get(this._editor)?.closeMessage();
 		this._currentPasteOperation?.cancel();
 		this._currentPasteOperation = undefined;
@@ -266,8 +260,7 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 		}
 
 		const metadata = this.fetchCopyMetadata(e);
-		const dataTransfer = toExternalVSDataTransfer(e.clipboardData);
-		dataTransfer.delete(vscodeClipboardMime);
+		this._logService.trace('CopyPasteController#handlePaste with metadata : ', metadata?.id, ' and text.length : ', e.clipboardData.getData('text/plain').length);
 
 		const fileTypes = Array.from(e.clipboardData.files).map(file => file.type);
 
@@ -300,8 +293,7 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 				this.showPasteAsNoEditMessage(selections, this._pasteAsActionContext.preferred);
 
 				// Also prevent default paste from applying
-				e.preventDefault();
-				e.stopImmediatePropagation();
+				e.setHandled();
 			}
 			return;
 		}
@@ -309,13 +301,12 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 		// Prevent the editor's default paste handler from running.
 		// Note that after this point, we are fully responsible for handling paste.
 		// If we can't provider a paste for any reason, we need to explicitly delegate pasting back to the editor.
-		e.preventDefault();
-		e.stopImmediatePropagation();
+		e.setHandled();
 
 		if (this._pasteAsActionContext) {
 			this.showPasteAsPick(this._pasteAsActionContext.preferred, allProviders, selections, dataTransfer, metadata);
 		} else {
-			this.doPasteInline(allProviders, selections, dataTransfer, metadata, e);
+			this.doPasteInline(allProviders, selections, dataTransfer, metadata, e.browserEvent);
 		}
 	}
 
@@ -329,7 +320,8 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 		MessageController.get(this._editor)?.showMessage(localize('pasteAsError', "No paste edits for '{0}' found", kindLabel), selections[0].getStartPosition());
 	}
 
-	private doPasteInline(allProviders: readonly DocumentPasteEditProvider[], selections: readonly Selection[], dataTransfer: VSDataTransfer, metadata: CopyMetadata | undefined, clipboardEvent: ClipboardEvent): void {
+	private doPasteInline(allProviders: readonly DocumentPasteEditProvider[], selections: readonly Selection[], dataTransfer: VSDataTransfer, metadata: CopyMetadata | undefined, clipboardEvent: ClipboardEvent | undefined): void {
+		this._logService.trace('CopyPasteController#doPasteInline');
 		const editor = this._editor;
 		if (!editor.hasModel()) {
 			return;
@@ -409,25 +401,21 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 
 		this._pasteProgressManager.showWhile(selections[0].getEndPosition(), localize('pasteIntoEditorProgress', "Running paste handlers. Click to cancel and do basic paste"), p, {
 			cancel: async () => {
-				try {
-					p.cancel();
-
-					if (editorStateCts.token.isCancellationRequested) {
-						return;
-					}
-
-					await this.applyDefaultPasteHandler(dataTransfer, metadata, editorStateCts.token, clipboardEvent);
-				} finally {
-					editorStateCts.dispose();
+				p.cancel();
+				if (editorStateCts.token.isCancellationRequested) {
+					return;
 				}
+
+				await this.applyDefaultPasteHandler(dataTransfer, metadata, editorStateCts.token, clipboardEvent);
 			}
-		}).then(() => {
+		}).finally(() => {
 			editorStateCts.dispose();
 		});
 		this._currentPasteOperation = p;
 	}
 
 	private showPasteAsPick(preference: PastePreference | undefined, allProviders: readonly DocumentPasteEditProvider[], selections: readonly Selection[], dataTransfer: VSDataTransfer, metadata: CopyMetadata | undefined): void {
+		this._logService.trace('CopyPasteController#showPasteAsPick');
 		const p = createCancelablePromise(async (token) => {
 			const editor = this._editor;
 			if (!editor.hasModel()) {
@@ -539,14 +527,13 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 		}, () => p);
 	}
 
-	private setCopyMetadata(dataTransfer: DataTransfer, metadata: CopyMetadata) {
-		dataTransfer.setData(vscodeClipboardMime, JSON.stringify(metadata));
+	private setCopyMetadata(clipboardData: IWritableClipboardData, metadata: CopyMetadata) {
+		this._logService.trace('CopyPasteController#setCopyMetadata new id : ', metadata.id);
+		clipboardData.setData(vscodeClipboardMime, JSON.stringify(metadata));
 	}
 
-	private fetchCopyMetadata(e: ClipboardEvent): CopyMetadata | undefined {
-		if (!e.clipboardData) {
-			return;
-		}
+	private fetchCopyMetadata(e: IClipboardPasteEvent): CopyMetadata | undefined {
+		this._logService.trace('CopyPasteController#fetchCopyMetadata');
 
 		// Prefer using the clipboard data we saved off
 		const rawMetadata = e.clipboardData.getData(vscodeClipboardMime);
@@ -558,14 +545,12 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 			}
 		}
 
-		// Otherwise try to extract the generic text editor metadata
-		const [_, metadata] = ClipboardEventUtils.getTextData(e.clipboardData);
-		if (metadata) {
+		if (e.metadata) {
 			return {
 				defaultPastePayload: {
-					mode: metadata.mode,
-					multicursorText: metadata.multicursorText ?? null,
-					pasteOnNewLine: !!metadata.isFromEmptySelection,
+					mode: e.metadata.mode,
+					multicursorText: e.metadata.multicursorText ?? null,
+					pasteOnNewLine: !!e.metadata.isFromEmptySelection,
 				},
 			};
 		}
@@ -574,6 +559,7 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 	}
 
 	private async mergeInDataFromCopy(allProviders: readonly DocumentPasteEditProvider[], dataTransfer: VSDataTransfer, metadata: CopyMetadata | undefined, token: CancellationToken): Promise<void> {
+		this._logService.trace('CopyPasteController#mergeInDataFromCopy with metadata : ', metadata?.id);
 		if (metadata?.id && CopyPasteController._currentCopyOperation?.handle === metadata.id) {
 			// Only resolve providers that have data we may care about
 			const toResolve = CopyPasteController._currentCopyOperation.operations
@@ -636,7 +622,7 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 		};
 	}
 
-	private async applyDefaultPasteHandler(dataTransfer: VSDataTransfer, metadata: CopyMetadata | undefined, token: CancellationToken, clipboardEvent: ClipboardEvent) {
+	private async applyDefaultPasteHandler(dataTransfer: VSDataTransfer, metadata: CopyMetadata | undefined, token: CancellationToken, clipboardEvent: ClipboardEvent | undefined) {
 		const textDataTransfer = dataTransfer.get(Mimes.text) ?? dataTransfer.get('text');
 		const text = (await textDataTransfer?.asString()) ?? '';
 		if (token.isCancellationRequested) {
@@ -650,6 +636,7 @@ export class CopyPasteController extends Disposable implements IEditorContributi
 			multicursorText: metadata?.defaultPastePayload.multicursorText ?? null,
 			mode: null,
 		};
+		this._logService.trace('CopyPasteController#applyDefaultPasteHandler for id : ', metadata?.id);
 		this._editor.trigger('keyboard', Handler.Paste, payload);
 	}
 
