@@ -9,6 +9,7 @@ import { cancelOnDispose } from '../../../../../base/common/cancellation.js';
 import { createHotClass } from '../../../../../base/common/hotReloadHelpers.js';
 import { Disposable, toDisposable } from '../../../../../base/common/lifecycle.js';
 import { ITransaction, autorun, derived, derivedDisposable, derivedObservableWithCache, observableFromEvent, observableSignal, observableValue, runOnChange, runOnChangeWithStore, transaction, waitForState } from '../../../../../base/common/observable.js';
+import { isEqual } from '../../../../../base/common/resources.js';
 import { isUndefined } from '../../../../../base/common/types.js';
 import { localize } from '../../../../../nls.js';
 import { IAccessibilityService } from '../../../../../platform/accessibility/common/accessibility.js';
@@ -22,6 +23,7 @@ import { hotClassGetOriginalInstance } from '../../../../../platform/observable/
 import { CoreEditingCommands } from '../../../../browser/coreCommands.js';
 import { ICodeEditor } from '../../../../browser/editorBrowser.js';
 import { observableCodeEditor } from '../../../../browser/observableCodeEditor.js';
+import { TriggerInlineEditCommandsRegistry } from '../../../../browser/triggerInlineEditCommandsRegistry.js';
 import { getOuterEditor } from '../../../../browser/widget/codeEditor/embeddedCodeEditorWidget.js';
 import { EditorOption } from '../../../../common/config/editorOptions.js';
 import { Position } from '../../../../common/core/position.js';
@@ -29,19 +31,22 @@ import { Range } from '../../../../common/core/range.js';
 import { CursorChangeReason } from '../../../../common/cursorEvents.js';
 import { ILanguageFeatureDebounceService } from '../../../../common/services/languageFeatureDebounce.js';
 import { ILanguageFeaturesService } from '../../../../common/services/languageFeatures.js';
+import { FIND_IDS } from '../../../find/browser/findModel.js';
+import { NextMarkerAction, NextMarkerInFilesAction, PrevMarkerAction, PrevMarkerInFilesAction } from '../../../gotoError/browser/gotoError.js';
+import { InsertLineAfterAction, InsertLineBeforeAction } from '../../../linesOperations/browser/linesOperations.js';
 import { InlineSuggestionHintsContentWidget } from '../hintsWidget/inlineCompletionsHintsWidget.js';
 import { TextModelChangeRecorder } from '../model/changeRecorder.js';
 import { InlineCompletionsModel } from '../model/inlineCompletionsModel.js';
 import { ObservableSuggestWidgetAdapter } from '../model/suggestWidgetAdapter.js';
 import { ObservableContextKeyService } from '../utils.js';
-import { InlineCompletionsView } from '../view/inlineCompletionsView.js';
+import { InlineSuggestionsView } from '../view/inlineSuggestionsView.js';
 import { inlineSuggestCommitId } from './commandIds.js';
 import { InlineCompletionContextKeys } from './inlineCompletionContextKeys.js';
 
 export class InlineCompletionsController extends Disposable {
 	private static readonly _instances = new Set<InlineCompletionsController>();
 
-	public static hot = createHotClass(InlineCompletionsController);
+	public static hot = createHotClass(this);
 	public static ID = 'editor.contrib.inlineCompletionsController';
 
 	/**
@@ -67,22 +72,50 @@ export class InlineCompletionsController extends Disposable {
 	private readonly _enabledInConfig;
 	private readonly _isScreenReaderEnabled;
 	private readonly _editorDictationInProgress;
-	private readonly _enabled;
+	private readonly _enabled = derived(this, reader => this._enabledInConfig.read(reader) && (!this._isScreenReaderEnabled.read(reader) || !this._editorDictationInProgress.read(reader)));
 
 	private readonly _debounceValue;
 
-	private readonly _focusIsInMenu;
-	private readonly _focusIsInEditorOrMenu;
+	private readonly _focusIsInMenu = observableValue<boolean>(this, false);
+	private readonly _focusIsInEditorOrMenu = derived(this, reader => {
+		const editorHasFocus = this._editorObs.isFocused.read(reader);
+		const menuHasFocus = this._focusIsInMenu.read(reader);
+		return editorHasFocus || menuHasFocus;
+	});
 
-	private readonly _cursorIsInIndentation;
+	private readonly _cursorIsInIndentation = derived(this, reader => {
+		const cursorPos = this._editorObs.cursorPosition.read(reader);
+		if (cursorPos === null) { return false; }
+		const model = this._editorObs.model.read(reader);
+		if (!model) { return false; }
+		this._editorObs.versionId.read(reader);
+		const indentMaxColumn = model.getLineIndentColumn(cursorPos.lineNumber);
+		return cursorPos.column <= indentMaxColumn;
+	});
 
-	public readonly model;
+	public readonly model = derivedDisposable<InlineCompletionsModel | undefined>(this, reader => {
+		if (this._editorObs.isReadonly.read(reader)) { return undefined; }
+		const textModel = this._editorObs.model.read(reader);
+		if (!textModel) { return undefined; }
 
-	private readonly _playAccessibilitySignal;
+		const model: InlineCompletionsModel = this._instantiationService.createInstance(
+			InlineCompletionsModel,
+			textModel,
+			this._suggestWidgetAdapter.selectedItem,
+			this._editorObs.versionId,
+			this._positions,
+			this._debounceValue,
+			this._enabled,
+			this.editor,
+		);
+		return model;
+	});
+
+	private readonly _playAccessibilitySignal = observableSignal(this);
 
 	private readonly _hideInlineEditOnSelectionChange;
 
-	protected readonly _view;
+	protected readonly _view = derived(reader => reader.store.add(this._instantiationService.createInstance(InlineSuggestionsView.hot.read(reader), this.editor, this.model, this._focusIsInMenu)));
 
 	constructor(
 		public readonly editor: ICodeEditor,
@@ -94,7 +127,7 @@ export class InlineCompletionsController extends Disposable {
 		@ILanguageFeaturesService private readonly _languageFeaturesService: ILanguageFeaturesService,
 		@IAccessibilitySignalService private readonly _accessibilitySignalService: IAccessibilitySignalService,
 		@IKeybindingService private readonly _keybindingService: IKeybindingService,
-		@IAccessibilityService private readonly _accessibilityService: IAccessibilityService,
+		@IAccessibilityService private readonly _accessibilityService: IAccessibilityService
 	) {
 		super();
 		this._editorObs = observableCodeEditor(this.editor);
@@ -110,47 +143,16 @@ export class InlineCompletionsController extends Disposable {
 			this._contextKeyService.onDidChangeContext,
 			() => this._contextKeyService.getContext(this.editor.getDomNode()).getValue('editorDictation.inProgress') === true
 		);
-		this._enabled = derived(this, reader => this._enabledInConfig.read(reader) && (!this._isScreenReaderEnabled.read(reader) || !this._editorDictationInProgress.read(reader)));
+
 		this._debounceValue = this._debounceService.for(
 			this._languageFeaturesService.inlineCompletionsProvider,
 			'InlineCompletionsDebounce',
 			{ min: 50, max: 50 }
 		);
-		this._focusIsInMenu = observableValue<boolean>(this, false);
-		this._focusIsInEditorOrMenu = derived(this, reader => {
-			const editorHasFocus = this._editorObs.isFocused.read(reader);
-			const menuHasFocus = this._focusIsInMenu.read(reader);
-			return editorHasFocus || menuHasFocus;
-		});
-		this._cursorIsInIndentation = derived(this, reader => {
-			const cursorPos = this._editorObs.cursorPosition.read(reader);
-			if (cursorPos === null) { return false; }
-			const model = this._editorObs.model.read(reader);
-			if (!model) { return false; }
-			this._editorObs.versionId.read(reader);
-			const indentMaxColumn = model.getLineIndentColumn(cursorPos.lineNumber);
-			return cursorPos.column <= indentMaxColumn;
-		});
-		this.model = derivedDisposable<InlineCompletionsModel | undefined>(this, reader => {
-			if (this._editorObs.isReadonly.read(reader)) { return undefined; }
-			const textModel = this._editorObs.model.read(reader);
-			if (!textModel) { return undefined; }
-
-			const model: InlineCompletionsModel = this._instantiationService.createInstance(
-				InlineCompletionsModel,
-				textModel,
-				this._suggestWidgetAdapter.selectedItem,
-				this._editorObs.versionId,
-				this._positions,
-				this._debounceValue,
-				this._enabled,
-				this.editor,
-			);
-			return model;
-		}).recomputeInitiallyAndOnChange(this._store);
-		this._playAccessibilitySignal = observableSignal(this);
+		this.model.recomputeInitiallyAndOnChange(this._store);
 		this._hideInlineEditOnSelectionChange = this._editorObs.getOption(EditorOption.inlineSuggest).map(val => true);
-		this._view = this._register(this._instantiationService.createInstance(InlineCompletionsView, this.editor, this.model, this._focusIsInMenu));
+
+		this._view.recomputeInitiallyAndOnChange(this._store);
 
 		InlineCompletionsController._instances.add(this);
 		this._register(toDisposable(() => InlineCompletionsController._instances.delete(this)));
@@ -159,14 +161,47 @@ export class InlineCompletionsController extends Disposable {
 			// Cancel all other inline completions when a new one starts
 			const model = this.model.read(reader);
 			if (!model) { return; }
-			if (model.state.read(reader) !== undefined) {
-				for (const ctrl of InlineCompletionsController._instances) {
-					if (ctrl !== this) {
-						ctrl.reject();
-					}
+			const state = model.state.read(reader);
+			if (!state) { return; }
+			if (!this._focusIsInEditorOrMenu.read(undefined)) { return; }
+
+			// This controller is in focus, hence reject others.
+			// However if we display a NES that relates to another edit then trigger NES on that related controller
+			const nextEditUri = state.kind === 'inlineEdit' ? state.nextEditUri : undefined;
+			for (const ctrl of InlineCompletionsController._instances) {
+				if (ctrl === this) {
+					continue;
+				} else if (nextEditUri && isEqual(nextEditUri, ctrl.editor.getModel()?.uri)) {
+					// The next edit in other edito is related to this controller, trigger it.
+					ctrl.model.read(undefined)?.trigger();
+				} else {
+					ctrl.reject();
 				}
 			}
 		}));
+		this._register(autorun(reader => {
+			// Cancel all other inline completions when a new one starts
+			const model = this.model.read(reader);
+			const uri = this.editor.getModel()?.uri;
+			if (!model || !uri) { return; }
+
+			// This NES was accepted, its possible there is an NES that points to this editor.
+			// I.e. there's an NES that reads `Go To Next Edit`,
+			// If there is one that points to this editor, then we need to hide that as this NES was accepted.
+			reader.store.add(model.onDidAccept(() => {
+				for (const ctrl of InlineCompletionsController._instances) {
+					if (ctrl === this) {
+						continue;
+					}
+					// Find the nes from another editor that points to this.
+					const state = ctrl.model.read(undefined)?.state.read(undefined);
+					if (state?.kind === 'inlineEdit' && isEqual(state.nextEditUri, uri)) {
+						ctrl.model.read(undefined)?.stop('automatic');
+					}
+				}
+			}));
+
+		}));
 
 		this._register(runOnChange(this._editorObs.onDidType, (_value, _changes) => {
 			if (this._enabled.get()) {
@@ -180,16 +215,24 @@ export class InlineCompletionsController extends Disposable {
 			}
 		}));
 
+		// These commands don't trigger onDidType.
+		const triggerCommands = new Set([
+			CoreEditingCommands.Tab.id,
+			CoreEditingCommands.DeleteLeft.id,
+			CoreEditingCommands.DeleteRight.id,
+			inlineSuggestCommitId,
+			'acceptSelectedSuggestion',
+			InsertLineAfterAction.ID,
+			InsertLineBeforeAction.ID,
+			FIND_IDS.NextMatchFindAction,
+			NextMarkerAction.ID,
+			PrevMarkerAction.ID,
+			NextMarkerInFilesAction.ID,
+			PrevMarkerInFilesAction.ID,
+			...TriggerInlineEditCommandsRegistry.getRegisteredCommands(),
+		]);
 		this._register(this._commandService.onDidExecuteCommand((e) => {
-			// These commands don't trigger onDidType.
-			const commands = new Set([
-				CoreEditingCommands.Tab.id,
-				CoreEditingCommands.DeleteLeft.id,
-				CoreEditingCommands.DeleteRight.id,
-				inlineSuggestCommitId,
-				'acceptSelectedSuggestion',
-			]);
-			if (commands.has(e.commandId) && editor.hasTextFocus() && this._enabled.get()) {
+			if (triggerCommands.has(e.commandId) && editor.hasTextFocus() && this._enabled.get()) {
 				let noDelay = false;
 				if (e.commandId === inlineSuggestCommitId) {
 					noDelay = true;
@@ -216,7 +259,20 @@ export class InlineCompletionsController extends Disposable {
 
 		this._register(autorun(reader => {
 			const isFocused = this._focusIsInEditorOrMenu.read(reader);
+			const model = this.model.read(undefined);
 			if (isFocused) {
+				// If this model already has an NES for another editor, then leave as is
+				// Else stop other models.
+				const state = model?.state.read(undefined);
+				if (!state || state.kind !== 'inlineEdit' || !state.nextEditUri) {
+					transaction(tx => {
+						for (const ctrl of InlineCompletionsController._instances) {
+							if (ctrl !== this) {
+								ctrl.model.read(undefined)?.stop('automatic', tx);
+							}
+						}
+					});
+				}
 				return;
 			}
 
@@ -228,9 +284,8 @@ export class InlineCompletionsController extends Disposable {
 				return;
 			}
 
-			const model = this.model.get();
 			if (!model) { return; }
-			if (model.state.get()?.inlineCompletion?.isFromExplicitRequest && model.inlineEditAvailable.get()) {
+			if (model.state.read(undefined)?.inlineSuggestion?.isFromExplicitRequest && model.inlineEditAvailable.read(undefined)) {
 				// dont hide inline edits on blur when requested explicitly
 				return;
 			}
@@ -262,7 +317,7 @@ export class InlineCompletionsController extends Disposable {
 			if (this._suggestWidgetAdapter.selectedItem.get()) {
 				return last;
 			}
-			return state?.inlineCompletion?.semanticId;
+			return state?.inlineSuggestion?.semanticId;
 		});
 		this._register(runOnChangeWithStore(derived(reader => {
 			this._playAccessibilitySignal.read(reader);
@@ -270,14 +325,18 @@ export class InlineCompletionsController extends Disposable {
 			return {};
 		}), async (_value, _, _deltas, store) => {
 			/** @description InlineCompletionsController.playAccessibilitySignalAndReadSuggestion */
-			const model = this.model.get();
-			const state = model?.state.get();
+			let model = this.model.get();
+			let state = model?.state.get();
 			if (!state || !model) { return; }
-			const lineText = state.kind === 'ghostText' ? model.textModel.getLineContent(state.primaryGhostText.lineNumber) : '';
 
 			await timeout(50, cancelOnDispose(store));
 			await waitForState(this._suggestWidgetAdapter.selectedItem, isUndefined, () => false, cancelOnDispose(store));
-			await this._accessibilitySignalService.playSignal(state.kind === 'ghostText' ? AccessibilitySignal.inlineSuggestion : AccessibilitySignal.nextEditSuggestion);
+
+			model = this.model.get();
+			state = model?.state.get();
+			if (!state || !model) { return; }
+			const lineText = state.kind === 'ghostText' ? model.textModel.getLineContent(state.primaryGhostText.lineNumber) : '';
+			this._accessibilitySignalService.playSignal(state.kind === 'ghostText' ? AccessibilitySignal.inlineSuggestion : AccessibilitySignal.nextEditSuggestion);
 
 			if (this.editor.getOption(EditorOption.screenReaderAnnounceInlineSuggestion)) {
 				if (state.kind === 'ghostText') {
@@ -313,12 +372,39 @@ export class InlineCompletionsController extends Disposable {
 		this._register(contextKeySvcObs.bind(InlineCompletionContextKeys.suppressSuggestions, reader => {
 			const model = this.model.read(reader);
 			const state = model?.inlineCompletionState.read(reader);
-			return state?.primaryGhostText && state?.inlineCompletion ? state.inlineCompletion.source.inlineSuggestions.suppressSuggestions : undefined;
+			return state?.primaryGhostText && state?.inlineSuggestion ? state.inlineSuggestion.source.inlineSuggestions.suppressSuggestions : undefined;
+		}));
+		this._register(contextKeySvcObs.bind(InlineCompletionContextKeys.inlineSuggestionAlternativeActionVisible, reader => {
+			const model = this.model.read(reader);
+			const state = model?.inlineEditState.read(reader);
+			const action = state?.inlineSuggestion.action;
+			return action && action.kind === 'edit' && action.alternativeAction !== undefined;
 		}));
 		this._register(contextKeySvcObs.bind(InlineCompletionContextKeys.inlineSuggestionVisible, reader => {
 			const model = this.model.read(reader);
 			const state = model?.inlineCompletionState.read(reader);
-			return !!state?.inlineCompletion && state?.primaryGhostText !== undefined && !state?.primaryGhostText.isEmpty();
+			return !!state?.inlineSuggestion && state?.primaryGhostText !== undefined && !state?.primaryGhostText.isEmpty();
+		}));
+		const firstGhostTextPos = derived(this, reader => {
+			const model = this.model.read(reader);
+			const state = model?.inlineCompletionState.read(reader);
+			const primaryGhostText = state?.primaryGhostText;
+			if (!primaryGhostText || primaryGhostText.isEmpty()) {
+				return undefined;
+			}
+			const firstPartPos = new Position(primaryGhostText.lineNumber, primaryGhostText.parts[0].column);
+			return firstPartPos;
+		});
+		this._register(contextKeySvcObs.bind(InlineCompletionContextKeys.cursorBeforeGhostText, reader => {
+			const firstPartPos = firstGhostTextPos.read(reader);
+			if (!firstPartPos) {
+				return false;
+			}
+			const cursorPos = this._editorObs.cursorPosition.read(reader);
+			if (!cursorPos) {
+				return false;
+			}
+			return firstPartPos.equals(cursorPos);
 		}));
 
 		this._register(this._instantiationService.createInstance(TextModelChangeRecorder, this.editor));
@@ -347,7 +433,7 @@ export class InlineCompletionsController extends Disposable {
 	}
 
 	public shouldShowHoverAtViewZone(viewZoneId: string): boolean {
-		return this._view.shouldShowHoverAtViewZone(viewZoneId);
+		return this._view.get().shouldShowHoverAtViewZone(viewZoneId);
 	}
 
 	public reject(): void {
@@ -355,6 +441,14 @@ export class InlineCompletionsController extends Disposable {
 			const m = this.model.get();
 			if (m) {
 				m.stop('explicitCancel', tx);
+				// Only if this controller is in focus can we cancel others.
+				if (this._focusIsInEditorOrMenu.get()) {
+					for (const ctrl of InlineCompletionsController._instances) {
+						if (ctrl !== this) {
+							ctrl.model.get()?.stop('automatic', tx);
+						}
+					}
+				}
 			}
 		});
 	}
@@ -365,8 +459,4 @@ export class InlineCompletionsController extends Disposable {
 			m.jump();
 		}
 	}
-
-	public testOnlyDisableUi() {
-		this._view.dispose();
-	}
 }
