@@ -30,7 +30,7 @@ import { registerIcon } from '../../../../../platform/theme/common/iconRegistry.
 import { ContentWidgetPositionPreference, ICodeEditor, IContentWidget, IContentWidgetPosition } from '../../../../browser/editorBrowser.js';
 import { EditorOption } from '../../../../common/config/editorOptions.js';
 import { Position } from '../../../../common/core/position.js';
-import { Command, InlineCompletionTriggerKind, InlineCompletionWarning } from '../../../../common/languages.js';
+import { InlineCompletionCommand, InlineCompletionTriggerKind, InlineCompletionWarning } from '../../../../common/languages.js';
 import { PositionAffinity } from '../../../../common/model.js';
 import { showNextInlineSuggestionActionId, showPreviousInlineSuggestionActionId } from '../controller/commandIds.js';
 import { InlineCompletionsModel } from '../model/inlineCompletionsModel.js';
@@ -118,7 +118,7 @@ const inlineSuggestionHintsNextIcon = registerIcon('inline-suggestion-hints-next
 const inlineSuggestionHintsPreviousIcon = registerIcon('inline-suggestion-hints-previous', Codicon.chevronLeft, localize('parameterHintsPreviousIcon', 'Icon for show previous parameter hint.'));
 
 export class InlineSuggestionHintsContentWidget extends Disposable implements IContentWidget {
-	public static readonly hot = createHotClass(InlineSuggestionHintsContentWidget);
+	public static readonly hot = createHotClass(this);
 
 	private static _dropDownVisible = false;
 	public static get dropDownVisible() { return this._dropDownVisible; }
@@ -143,12 +143,7 @@ export class InlineSuggestionHintsContentWidget extends Disposable implements IC
 			true,
 			() => this._commandService.executeCommand(commandId),
 		);
-		const kb = this.keybindingService.lookupKeybinding(commandId, this._contextKeyService);
-		let tooltip = label;
-		if (kb) {
-			tooltip = localize({ key: 'content', comment: ['A label', 'A keybinding'] }, '{0} ({1})', label, kb.getLabel());
-		}
-		action.tooltip = tooltip;
+		action.tooltip = this.keybindingService.appendKeybinding(label, commandId, this._contextKeyService);
 		return action;
 	}
 
@@ -171,7 +166,7 @@ export class InlineSuggestionHintsContentWidget extends Disposable implements IC
 		private readonly _position: IObservable<Position | null>,
 		private readonly _currentSuggestionIdx: IObservable<number>,
 		private readonly _suggestionCount: IObservable<number | undefined>,
-		private readonly _extraCommands: IObservable<Command[]>,
+		private readonly _extraCommands: IObservable<InlineCompletionCommand[]>,
 		private readonly _warning: IObservable<InlineCompletionWarning | undefined>,
 		private readonly _relayout: () => void,
 		@ICommandService private readonly _commandService: ICommandService,
@@ -289,12 +284,12 @@ export class InlineSuggestionHintsContentWidget extends Disposable implements IC
 			const extraCommands = this._extraCommands.read(reader);
 			const extraActions = extraCommands.map<IAction>(c => ({
 				class: undefined,
-				id: c.id,
+				id: c.command.id,
 				enabled: true,
-				tooltip: c.tooltip || '',
-				label: c.title,
+				tooltip: c.command.tooltip || '',
+				label: c.command.title,
 				run: (event) => {
-					return this._commandService.executeCommand(c.id);
+					return this._commandService.executeCommand(c.command.id);
 				},
 			}));
 
