@@ -18,35 +18,35 @@ suite('PositionOffsetTransformer', () => {
 		assert.deepStrictEqual(
 			new OffsetRange(0, str.length + 2).map(i => t.getPosition(i).toString()),
 			[
-				"(1,1)",
-				"(1,2)",
-				"(1,3)",
-				"(1,4)",
-				"(1,5)",
-				"(1,6)",
-				"(1,7)",
-				"(2,1)",
-				"(2,2)",
-				"(2,3)",
-				"(2,4)",
-				"(2,5)",
-				"(2,6)",
-				"(2,7)",
-				"(3,1)",
-				"(3,2)",
-				"(3,3)",
-				"(3,4)",
-				"(3,5)",
-				"(3,6)",
-				"(3,7)",
-				"(4,1)",
-				"(4,2)",
-				"(4,3)",
-				"(4,4)",
-				"(4,5)",
-				"(4,6)",
-				"(4,7)",
-				"(4,8)"
+				'(1,1)',
+				'(1,2)',
+				'(1,3)',
+				'(1,4)',
+				'(1,5)',
+				'(1,6)',
+				'(1,7)',
+				'(2,1)',
+				'(2,2)',
+				'(2,3)',
+				'(2,4)',
+				'(2,5)',
+				'(2,6)',
+				'(2,7)',
+				'(3,1)',
+				'(3,2)',
+				'(3,3)',
+				'(3,4)',
+				'(3,5)',
+				'(3,6)',
+				'(3,7)',
+				'(4,1)',
+				'(4,2)',
+				'(4,3)',
+				'(4,4)',
+				'(4,5)',
+				'(4,6)',
+				'(4,7)',
+				'(4,8)'
 			]
 		);
 	});
