@@ -17,6 +17,8 @@ import {
 import { ExtensionType, IExtensionManifest, TargetPlatform } from '../../extensions/common/extensions.js';
 import { IProductService } from '../../product/common/productService.js';
 import { CommontExtensionManagementService } from './abstractExtensionManagementService.js';
+import { language } from '../../../base/common/platform.js';
+import { RemoteAgentConnectionContext } from '../../remote/common/remoteAgentEnvironment.js';
 
 function transformIncomingURI(uri: UriComponents, transformer: IURITransformer | null): URI;
 function transformIncomingURI(uri: UriComponents | undefined, transformer: IURITransformer | null): URI | undefined;
@@ -43,15 +45,15 @@ function transformOutgoingExtension(extension: ILocalExtension, transformer: IUR
 	return transformer ? cloneAndChange(extension, value => value instanceof URI ? transformer.transformOutgoingURI(value) : undefined) : extension;
 }
 
-export class ExtensionManagementChannel implements IServerChannel {
+export class ExtensionManagementChannel<TContext = RemoteAgentConnectionContext | string> implements IServerChannel<TContext> {
 
-	onInstallExtension: Event<InstallExtensionEvent>;
-	onDidInstallExtensions: Event<readonly InstallExtensionResult[]>;
-	onUninstallExtension: Event<UninstallExtensionEvent>;
-	onDidUninstallExtension: Event<DidUninstallExtensionEvent>;
-	onDidUpdateExtensionMetadata: Event<DidUpdateExtensionMetadata>;
+	readonly onInstallExtension: Event<InstallExtensionEvent>;
+	readonly onDidInstallExtensions: Event<readonly InstallExtensionResult[]>;
+	readonly onUninstallExtension: Event<UninstallExtensionEvent>;
+	readonly onDidUninstallExtension: Event<DidUninstallExtensionEvent>;
+	readonly onDidUpdateExtensionMetadata: Event<DidUpdateExtensionMetadata>;
 
-	constructor(private service: IExtensionManagementService, private getUriTransformer: (requestContext: any) => IURITransformer | null) {
+	constructor(private service: IExtensionManagementService, private getUriTransformer: (requestContext: TContext) => IURITransformer | null) {
 		this.onInstallExtension = Event.buffer(service.onInstallExtension, true);
 		this.onDidInstallExtensions = Event.buffer(service.onDidInstallExtensions, true);
 		this.onUninstallExtension = Event.buffer(service.onUninstallExtension, true);
@@ -59,6 +61,7 @@ export class ExtensionManagementChannel implements IServerChannel {
 		this.onDidUpdateExtensionMetadata = Event.buffer(service.onDidUpdateExtensionMetadata, true);
 	}
 
+	// eslint-disable-next-line @typescript-eslint/no-explicit-any
 	listen(context: any, event: string): Event<any> {
 		const uriTransformer = this.getUriTransformer(context);
 		switch (event) {
@@ -107,6 +110,7 @@ export class ExtensionManagementChannel implements IServerChannel {
 		throw new Error('Invalid listen');
 	}
 
+	// eslint-disable-next-line @typescript-eslint/no-explicit-any
 	async call(context: any, command: string, args?: any): Promise<any> {
 		const uriTransformer: IURITransformer | null = this.getUriTransformer(context);
 		switch (command) {
@@ -145,7 +149,7 @@ export class ExtensionManagementChannel implements IServerChannel {
 				return this.service.uninstallExtensions(arg.map(({ extension, options }) => ({ extension: transformIncomingExtension(extension, uriTransformer), options: transformIncomingOptions(options, uriTransformer) })));
 			}
 			case 'getInstalled': {
-				const extensions = await this.service.getInstalled(args[0], transformIncomingURI(args[1], uriTransformer), args[2]);
+				const extensions = await this.service.getInstalled(args[0], transformIncomingURI(args[1], uriTransformer), args[2], args[3]);
 				return extensions.map(e => transformOutgoingExtension(e, uriTransformer));
 			}
 			case 'toggleApplicationScope': {
@@ -235,12 +239,13 @@ export class ExtensionManagementChannelClient extends CommontExtensionManagement
 		this._onDidUpdateExtensionMetadata.fire(event);
 	}
 
-	private isUriComponents(thing: unknown): thing is UriComponents {
-		if (!thing) {
+	private isUriComponents(obj: unknown): obj is UriComponents {
+		if (!obj) {
 			return false;
 		}
-		return typeof (<any>thing).path === 'string' &&
-			typeof (<any>thing).scheme === 'string';
+		const thing = obj as UriComponents | undefined;
+		return typeof thing?.path === 'string' &&
+			typeof thing?.scheme === 'string';
 	}
 
 	protected _targetPlatformPromise: Promise<TargetPlatform> | undefined;
@@ -297,7 +302,7 @@ export class ExtensionManagementChannelClient extends CommontExtensionManagement
 	}
 
 	getInstalled(type: ExtensionType | null = null, extensionsProfileResource?: URI, productVersion?: IProductVersion): Promise<ILocalExtension[]> {
-		return Promise.resolve(this.channel.call<ILocalExtension[]>('getInstalled', [type, extensionsProfileResource, productVersion]))
+		return Promise.resolve(this.channel.call<ILocalExtension[]>('getInstalled', [type, extensionsProfileResource, productVersion, language]))
 			.then(extensions => extensions.map(extension => transformIncomingExtension(extension, null)));
 	}
 
@@ -340,10 +345,12 @@ export class ExtensionTipsChannel implements IServerChannel {
 	constructor(private service: IExtensionTipsService) {
 	}
 
+	// eslint-disable-next-line @typescript-eslint/no-explicit-any
 	listen(context: any, event: string): Event<any> {
 		throw new Error('Invalid listen');
 	}
 
+	// eslint-disable-next-line @typescript-eslint/no-explicit-any
 	call(context: any, command: string, args?: any): Promise<any> {
 		switch (command) {
 			case 'getConfigBasedTips': return this.service.getConfigBasedTips(URI.revive(args[0]));
