@@ -15,12 +15,12 @@ import { URI } from '../../../base/common/uri.js';
 import { localize } from '../../../nls.js';
 import { ILogService, LogLevel } from '../../log/common/log.js';
 import { IProductService } from '../../product/common/productService.js';
-import { FlowControlConstants, IShellLaunchConfig, ITerminalChildProcess, ITerminalLaunchError, IProcessProperty, IProcessPropertyMap as IProcessPropertyMap, ProcessPropertyType, TerminalShellType, IProcessReadyEvent, ITerminalProcessOptions, PosixShellType, IProcessReadyWindowsPty, GeneralShellType } from '../common/terminal.js';
+import { FlowControlConstants, IShellLaunchConfig, ITerminalChildProcess, ITerminalLaunchError, IProcessProperty, IProcessPropertyMap, ProcessPropertyType, TerminalShellType, IProcessReadyEvent, ITerminalProcessOptions, PosixShellType, IProcessReadyWindowsPty, GeneralShellType, ITerminalLaunchResult } from '../common/terminal.js';
 import { ChildProcessMonitor } from './childProcessMonitor.js';
 import { getShellIntegrationInjection, getWindowsBuildNumber, IShellIntegrationConfigInjection } from './terminalEnvironment.js';
 import { WindowsShellHelper } from './windowsShellHelper.js';
 import { IPty, IPtyForkOptions, IWindowsPtyForkOptions, spawn } from 'node-pty';
-import { chunkInput } from '../common/terminalProcess.js';
+import { isNumber } from '../../../base/common/types.js';
 
 const enum ShutdownConstants {
 	/**
@@ -56,15 +56,6 @@ const enum Constants {
 	 * interval.
 	 */
 	KillSpawnSpacingDuration = 50,
-	/**
-	 * How long to wait between chunk writes.
-	 */
-	WriteInterval = 5,
-}
-
-interface IWriteObject {
-	data: string;
-	isBinary: boolean;
 }
 
 const posixShellTypeMap = new Map<string, PosixShellType>([
@@ -83,7 +74,7 @@ const generalShellTypeMap = new Map<string, GeneralShellType>([
 	['julia', GeneralShellType.Julia],
 	['nu', GeneralShellType.NuShell],
 	['node', GeneralShellType.Node],
-
+	['xonsh', GeneralShellType.Xonsh],
 ]);
 export class TerminalProcess extends Disposable implements ITerminalChildProcess {
 	readonly id = 0;
@@ -112,8 +103,6 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 	private _windowsShellHelper: WindowsShellHelper | undefined;
 	private _childProcessMonitor: ChildProcessMonitor | undefined;
 	private _titleInterval: Timeout | undefined;
-	private _writeQueue: IWriteObject[] = [];
-	private _writeTimeout: Timeout | undefined;
 	private _delayedResizer: DelayedResizer | undefined;
 	private readonly _initialCwd: string;
 	private readonly _ptyOptions: IPtyForkOptions | IWindowsPtyForkOptions;
@@ -130,7 +119,7 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 	readonly onProcessData = this._onProcessData.event;
 	private readonly _onProcessReady = this._register(new Emitter<IProcessReadyEvent>());
 	readonly onProcessReady = this._onProcessReady.event;
-	private readonly _onDidChangeProperty = this._register(new Emitter<IProcessProperty<any>>());
+	private readonly _onDidChangeProperty = this._register(new Emitter<IProcessProperty>());
 	readonly onDidChangeProperty = this._onDidChangeProperty.event;
 	private readonly _onProcessExit = this._register(new Emitter<number>());
 	readonly onProcessExit = this._onProcessExit.event;
@@ -161,7 +150,7 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 		this._initialCwd = cwd;
 		this._properties[ProcessPropertyType.InitialCwd] = this._initialCwd;
 		this._properties[ProcessPropertyType.Cwd] = this._initialCwd;
-		const useConpty = this._options.windowsEnableConpty && process.platform === 'win32' && getWindowsBuildNumber() >= 18309;
+		const useConpty = process.platform === 'win32' && getWindowsBuildNumber() >= 18309;
 		const useConptyDll = useConpty && this._options.windowsUseConptyDll;
 		this._ptyOptions = {
 			name,
@@ -178,7 +167,7 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 		// Delay resizes to avoid conpty not respecting very early resize calls
 		if (isWindows) {
 			if (useConpty && cols === 0 && rows === 0 && this.shellLaunchConfig.executable?.endsWith('Git\\bin\\bash.exe')) {
-				this._delayedResizer = new DelayedResizer();
+				this._delayedResizer = this._register(new DelayedResizer());
 				this._register(this._delayedResizer.onTrigger(dimensions => {
 					this._delayedResizer?.dispose();
 					this._delayedResizer = undefined;
@@ -188,11 +177,11 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 				}));
 			}
 			// WindowsShellHelper is used to fetch the process title and shell type
-			this.onProcessReady(e => {
+			this._register(this.onProcessReady(e => {
 				this._windowsShellHelper = this._register(new WindowsShellHelper(e.pid));
 				this._register(this._windowsShellHelper.onShellTypeChanged(e => this._onDidChangeProperty.fire({ type: ProcessPropertyType.ShellType, value: e })));
 				this._register(this._windowsShellHelper.onShellNameChanged(e => this._onDidChangeProperty.fire({ type: ProcessPropertyType.Title, value: e })));
-			});
+			}));
 		}
 		this._register(toDisposable(() => {
 			if (this._titleInterval) {
@@ -200,9 +189,13 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 				this._titleInterval = undefined;
 			}
 		}));
+		this._register(toDisposable(() => {
+			this._ptyProcess = undefined;
+			this._processStartupComplete = undefined;
+		}));
 	}
 
-	async start(): Promise<ITerminalLaunchError | { injectedArgs: string[] } | undefined> {
+	async start(): Promise<ITerminalLaunchError | ITerminalLaunchResult | undefined> {
 		const results = await Promise.all([this._validateCwd(), this._validateExecutable()]);
 		const firstError = results.find(r => r !== undefined);
 		if (firstError) {
@@ -234,6 +227,12 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 		} else {
 			this._onDidChangeProperty.fire({ type: ProcessPropertyType.FailedShellIntegrationActivation, value: true });
 			this._onDidChangeProperty.fire({ type: ProcessPropertyType.ShellIntegrationInjectionFailureReason, value: injection.reason });
+			// Even if shell integration injection failed, still set the nonce if one was provided
+			// This allows extensions to use shell integration with custom shells
+			if (this._options.shellIntegration.nonce) {
+				this._ptyOptions.env ||= {};
+				this._ptyOptions.env['VSCODE_NONCE'] = this._options.shellIntegration.nonce;
+			}
 		}
 
 		try {
@@ -306,11 +305,11 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 		const ptyProcess = spawn(shellLaunchConfig.executable!, args, options);
 		this._ptyProcess = ptyProcess;
 		this._childProcessMonitor = this._register(new ChildProcessMonitor(ptyProcess.pid, this._logService));
-		this._childProcessMonitor.onDidChangeHasChildProcesses(value => this._onDidChangeProperty.fire({ type: ProcessPropertyType.HasChildProcesses, value }));
+		this._register(this._childProcessMonitor.onDidChangeHasChildProcesses(value => this._onDidChangeProperty.fire({ type: ProcessPropertyType.HasChildProcesses, value })));
 		this._processStartupComplete = new Promise<void>(c => {
-			this.onProcessReady(() => c());
+			this._register(this.onProcessReady(() => c()));
 		});
-		ptyProcess.onData(data => {
+		this._register(ptyProcess.onData(data => {
 			// Handle flow control
 			this._unacknowledgedCharCount += data.length;
 			if (!this._isPtyPaused && this._unacknowledgedCharCount > FlowControlConstants.HighWatermarkChars) {
@@ -327,11 +326,11 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 			}
 			this._windowsShellHelper?.checkShell();
 			this._childProcessMonitor?.handleOutput();
-		});
-		ptyProcess.onExit(e => {
+		}));
+		this._register(ptyProcess.onExit(e => {
 			this._exitCode = e.exitCode;
 			this._queueProcessExit();
-		});
+		}));
 		this._sendProcessId(ptyProcess.pid);
 		this._setupTitlePolling(ptyProcess);
 	}
@@ -388,7 +387,7 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 
 	private async _throttleKillSpawn(): Promise<void> {
 		// Only throttle on Windows/conpty
-		if (!isWindows || !('useConpty' in this._ptyOptions) || !this._ptyOptions.useConpty) {
+		if (!isWindows || !hasConptyOption(this._ptyOptions) || !this._ptyOptions.useConpty) {
 			return;
 		}
 		// Don't throttle when using conpty.dll as it seems to have been fixed in later versions
@@ -460,13 +459,20 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 	}
 
 	input(data: string, isBinary: boolean = false): void {
+		this._logService.trace('node-pty.IPty#write', data, isBinary);
+		if (isBinary) {
+			this._ptyProcess!.write(Buffer.from(data, 'binary'));
+		} else {
+			this._ptyProcess!.write(data);
+		}
+		this._childProcessMonitor?.handleInput();
+	}
+
+	sendSignal(signal: string): void {
 		if (this._store.isDisposed || !this._ptyProcess) {
 			return;
 		}
-		this._writeQueue.push(...chunkInput(data).map(e => {
-			return { isBinary, data: e };
-		}));
-		this._startWrite();
+		this._ptyProcess.kill(signal);
 	}
 
 	async processBinary(data: string): Promise<void> {
@@ -504,47 +510,15 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 		}
 	}
 
-	private _startWrite(): void {
-		// Don't write if it's already queued of is there is nothing to write
-		if (this._writeTimeout !== undefined || this._writeQueue.length === 0) {
-			return;
-		}
-
-		this._doWrite();
-
-		// Don't queue more writes if the queue is empty
-		if (this._writeQueue.length === 0) {
-			this._writeTimeout = undefined;
-			return;
-		}
-
-		// Queue the next write
-		this._writeTimeout = setTimeout(() => {
-			this._writeTimeout = undefined;
-			this._startWrite();
-		}, Constants.WriteInterval);
-	}
-
-	private _doWrite(): void {
-		const object = this._writeQueue.shift()!;
-		this._logService.trace('node-pty.IPty#write', object.data);
-		if (object.isBinary) {
-			this._ptyProcess!.write(Buffer.from(object.data, 'binary') as any);
-		} else {
-			this._ptyProcess!.write(object.data);
-		}
-		this._childProcessMonitor?.handleInput();
-	}
-
 	resize(cols: number, rows: number): void {
 		if (this._store.isDisposed) {
 			return;
 		}
-		if (typeof cols !== 'number' || typeof rows !== 'number' || isNaN(cols) || isNaN(rows)) {
+		if (!isNumber(cols) || !isNumber(rows)) {
 			return;
 		}
-		// Ensure that cols and rows are always >= 1, this prevents a native
-		// exception in winpty.
+		// Ensure that cols and rows are always >= 1, this prevents a native exception in winpty.
+		// TODO: Handle this directly on node-pty instead: https://github.com/microsoft/node-pty/issues/877
 		if (this._ptyProcess) {
 			cols = Math.max(cols, 1);
 			rows = Math.max(rows, 1);
@@ -642,7 +616,7 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 
 	getWindowsPty(): IProcessReadyWindowsPty | undefined {
 		return isWindows ? {
-			backend: 'useConpty' in this._ptyOptions && this._ptyOptions.useConpty ? 'conpty' : 'winpty',
+			backend: 'conpty',
 			buildNumber: getWindowsBuildNumber()
 		} : undefined;
 	}
@@ -667,3 +641,7 @@ class DelayedResizer extends Disposable {
 		this._register(toDisposable(() => clearTimeout(this._timeout)));
 	}
 }
+
+function hasConptyOption(obj: IPtyForkOptions | IWindowsPtyForkOptions): obj is IWindowsPtyForkOptions {
+	return 'useConpty' in obj;
+}
