@@ -8,10 +8,10 @@
 import { URI, UriComponents } from '../../../base/common/uri.js';
 import { Event, Emitter } from '../../../base/common/event.js';
 import { debounce } from '../../../base/common/decorators.js';
-import { DisposableStore, IDisposable, MutableDisposable } from '../../../base/common/lifecycle.js';
+import { DisposableMap, DisposableStore, IDisposable, MutableDisposable } from '../../../base/common/lifecycle.js';
 import { asPromise } from '../../../base/common/async.js';
 import { ExtHostCommands } from './extHostCommands.js';
-import { MainContext, MainThreadSCMShape, SCMRawResource, SCMRawResourceSplice, SCMRawResourceSplices, IMainContext, ExtHostSCMShape, ICommandDto, MainThreadTelemetryShape, SCMGroupFeatures, SCMHistoryItemDto, SCMHistoryItemChangeDto, SCMHistoryItemRefDto, SCMActionButtonDto } from './extHost.protocol.js';
+import { MainContext, MainThreadSCMShape, SCMRawResource, SCMRawResourceSplice, SCMRawResourceSplices, IMainContext, ExtHostSCMShape, ICommandDto, MainThreadTelemetryShape, SCMGroupFeatures, SCMHistoryItemDto, SCMHistoryItemChangeDto, SCMHistoryItemRefDto, SCMActionButtonDto, SCMArtifactGroupDto, SCMArtifactDto } from './extHost.protocol.js';
 import { sortedDiff, equals } from '../../../base/common/arrays.js';
 import { comparePaths } from '../../../base/common/comparers.js';
 import type * as vscode from 'vscode';
@@ -22,12 +22,13 @@ import { ExtensionIdentifierMap, IExtensionDescription } from '../../../platform
 import { MarshalledId } from '../../../base/common/marshallingIds.js';
 import { ThemeIcon } from '../../../base/common/themables.js';
 import { IMarkdownString } from '../../../base/common/htmlContent.js';
-import { MarkdownString } from './extHostTypeConverters.js';
+import { MarkdownString, SourceControlInputBoxValidationType } from './extHostTypeConverters.js';
 import { checkProposedApiEnabled, isProposedApiEnabled } from '../../services/extensions/common/extensions.js';
 import { ExtHostDocuments } from './extHostDocuments.js';
 import { Schemas } from '../../../base/common/network.js';
 import { isLinux } from '../../../base/common/platform.js';
 import { structuralEquals } from '../../../base/common/equals.js';
+import { Iterable } from '../../../base/common/iterator.js';
 
 type ProviderHandle = number;
 type GroupHandle = number;
@@ -74,11 +75,15 @@ function getHistoryItemIconDto(icon: vscode.Uri | { light: vscode.Uri; dark: vsc
 
 function toSCMHistoryItemDto(historyItem: vscode.SourceControlHistoryItem): SCMHistoryItemDto {
 	const authorIcon = getHistoryItemIconDto(historyItem.authorIcon);
+	const tooltip = Array.isArray(historyItem.tooltip)
+		? MarkdownString.fromMany(historyItem.tooltip)
+		: historyItem.tooltip ? MarkdownString.from(historyItem.tooltip) : undefined;
+
 	const references = historyItem.references?.map(r => ({
 		...r, icon: getHistoryItemIconDto(r.icon)
 	}));
 
-	return { ...historyItem, authorIcon, references };
+	return { ...historyItem, authorIcon, references, tooltip };
 }
 
 function toSCMHistoryItemRefDto(historyItemRef?: vscode.SourceControlHistoryItemRef): SCMHistoryItemRefDto | undefined {
@@ -369,8 +374,7 @@ export class ExtHostSCMInputBox implements vscode.SourceControlInputBox {
 
 	showValidationMessage(message: string | vscode.MarkdownString, type: vscode.SourceControlInputBoxValidationType) {
 		checkProposedApiEnabled(this._extension, 'scmValidation');
-
-		this.#proxy.$showValidationMessage(this._sourceControlHandle, message, type as any);
+		this.#proxy.$showValidationMessage(this._sourceControlHandle, message, SourceControlInputBoxValidationType.from(type));
 	}
 
 	$onInputBoxValueChange(value: string): void {
@@ -542,6 +546,12 @@ class ExtHostSourceControl implements vscode.SourceControl {
 
 	private static _handlePool: number = 0;
 
+	readonly onDidDisposeParent: Event<void>;
+
+	private readonly _onDidDispose = new Emitter<void>();
+	readonly onDidDispose = this._onDidDispose.event;
+
+
 	#proxy: MainThreadSCMShape;
 
 	private _groups: Map<GroupHandle, ExtHostSourceControlResourceGroup> = new Map<GroupHandle, ExtHostSourceControlResourceGroup>();
@@ -558,6 +568,24 @@ class ExtHostSourceControl implements vscode.SourceControl {
 		return this._rootUri;
 	}
 
+	private _contextValue: string | undefined = undefined;
+
+	get contextValue(): string | undefined {
+		checkProposedApiEnabled(this._extension, 'scmProviderOptions');
+		return this._contextValue;
+	}
+
+	set contextValue(contextValue: string | undefined) {
+		checkProposedApiEnabled(this._extension, 'scmProviderOptions');
+
+		if (this._contextValue === contextValue) {
+			return;
+		}
+
+		this._contextValue = contextValue;
+		this.#proxy.$updateSourceControl(this.handle, { contextValue });
+	}
+
 	private _inputBox: ExtHostSCMInputBox;
 	get inputBox(): ExtHostSCMInputBox { return this._inputBox; }
 
@@ -644,6 +672,31 @@ class ExtHostSourceControl implements vscode.SourceControl {
 		}
 	}
 
+	private _artifactProvider: vscode.SourceControlArtifactProvider | undefined;
+	private readonly _artifactProviderDisposable = new MutableDisposable<DisposableStore>();
+
+	get artifactProvider(): vscode.SourceControlArtifactProvider | undefined {
+		checkProposedApiEnabled(this._extension, 'scmArtifactProvider');
+		return this._artifactProvider;
+	}
+
+	set artifactProvider(artifactProvider: vscode.SourceControlArtifactProvider | undefined) {
+		checkProposedApiEnabled(this._extension, 'scmArtifactProvider');
+
+		this._artifactProvider = artifactProvider;
+		this._artifactProviderDisposable.value = new DisposableStore();
+
+		this.#proxy.$updateSourceControl(this.handle, { hasArtifactProvider: !!artifactProvider });
+
+		if (artifactProvider) {
+			this._artifactProviderDisposable.value.add(artifactProvider.onDidChangeArtifacts((groups: string[]) => {
+				if (groups.length !== 0) {
+					this.#proxy.$onDidChangeArtifacts(this.handle, groups);
+				}
+			}));
+		}
+	}
+
 	private _commitTemplate: string | undefined = undefined;
 
 	get commitTemplate(): string | undefined {
@@ -675,7 +728,9 @@ class ExtHostSourceControl implements vscode.SourceControl {
 		this.#proxy.$updateSourceControl(this.handle, { acceptInputCommand: internal });
 	}
 
-	private readonly _actionButtonDisposables = new MutableDisposable<DisposableStore>();
+	// We know what we're doing here:
+	// eslint-disable-next-line local/code-no-potentially-unsafe-disposables
+	private _actionButtonDisposables = new DisposableStore();
 	private _actionButton: vscode.SourceControlActionButton | undefined;
 	get actionButton(): vscode.SourceControlActionButton | undefined {
 		checkProposedApiEnabled(this._extension, 'scmActionButton');
@@ -692,26 +747,32 @@ class ExtHostSourceControl implements vscode.SourceControl {
 			return;
 		}
 
+		// In order to prevent disposing the action button command that are still rendered in the UI
+		// until the next UI update, we ensure to dispose them after the update has been completed.
+		const oldActionButtonDisposables = this._actionButtonDisposables;
+		this._actionButtonDisposables = new DisposableStore();
+
 		this._actionButton = actionButton;
-		this._actionButtonDisposables.value = new DisposableStore();
 
 		const actionButtonDto = actionButton !== undefined ?
 			{
 				command: {
-					...this._commands.converter.toInternal(actionButton.command, this._actionButtonDisposables.value),
+					...this._commands.converter.toInternal(actionButton.command, this._actionButtonDisposables),
 					shortTitle: actionButton.command.shortTitle
 				},
 				secondaryCommands: actionButton.secondaryCommands?.map(commandGroup => {
-					return commandGroup.map(command => this._commands.converter.toInternal(command, this._actionButtonDisposables.value!));
+					return commandGroup.map(command => this._commands.converter.toInternal(command, this._actionButtonDisposables));
 				}),
 				enabled: actionButton.enabled
-			} satisfies SCMActionButtonDto : undefined;
+			} satisfies SCMActionButtonDto : null;
 
-		this.#proxy.$updateSourceControl(this.handle, { actionButton: actionButtonDto ?? null });
+		this.#proxy.$updateSourceControl(this.handle, { actionButton: actionButtonDto })
+			.finally(() => oldActionButtonDisposables.dispose());
 	}
 
-
-	private readonly _statusBarDisposables = new MutableDisposable<DisposableStore>();
+	// We know what we're doing here:
+	// eslint-disable-next-line local/code-no-potentially-unsafe-disposables
+	private _statusBarDisposables = new DisposableStore();
 	private _statusBarCommands: vscode.Command[] | undefined = undefined;
 
 	get statusBarCommands(): vscode.Command[] | undefined {
@@ -723,12 +784,17 @@ class ExtHostSourceControl implements vscode.SourceControl {
 			return;
 		}
 
-		this._statusBarDisposables.value = new DisposableStore();
+		// In order to prevent disposing status bar commands that are still rendered in the UI
+		// until the next UI update, we ensure to dispose them after the update has been completed.
+		const oldStatusBarDisposables = this._statusBarDisposables;
+		this._statusBarDisposables = new DisposableStore();
 
 		this._statusBarCommands = statusBarCommands;
 
-		const internal = (statusBarCommands || []).map(c => this._commands.converter.toInternal(c, this._statusBarDisposables.value!)) as ICommandDto[];
-		this.#proxy.$updateSourceControl(this.handle, { statusBarCommands: internal });
+		const internal = (statusBarCommands || []).map(c => this._commands.converter.toInternal(c, this._statusBarDisposables)) as ICommandDto[];
+
+		this.#proxy.$updateSourceControl(this.handle, { statusBarCommands: internal })
+			.finally(() => oldStatusBarDisposables.dispose());
 	}
 
 	private _selected: boolean = false;
@@ -740,7 +806,9 @@ class ExtHostSourceControl implements vscode.SourceControl {
 	private readonly _onDidChangeSelection = new Emitter<boolean>();
 	readonly onDidChangeSelection = this._onDidChangeSelection.event;
 
-	private handle: number = ExtHostSourceControl._handlePool++;
+	private readonly _artifactCommandsDisposables = new DisposableMap<string /* artifact group */, DisposableStore>();
+
+	readonly handle: number = ExtHostSourceControl._handlePool++;
 
 	constructor(
 		private readonly _extension: IExtensionDescription,
@@ -749,7 +817,10 @@ class ExtHostSourceControl implements vscode.SourceControl {
 		private _commands: ExtHostCommands,
 		private _id: string,
 		private _label: string,
-		private _rootUri?: vscode.Uri
+		private _rootUri?: vscode.Uri,
+		_iconPath?: vscode.IconPath,
+		_isHidden?: boolean,
+		_parent?: ExtHostSourceControl
 	) {
 		this.#proxy = proxy;
 
@@ -760,7 +831,9 @@ class ExtHostSourceControl implements vscode.SourceControl {
 		});
 
 		this._inputBox = new ExtHostSCMInputBox(_extension, _extHostDocuments, this.#proxy, this.handle, inputBoxDocumentUri);
-		this.#proxy.$registerSourceControl(this.handle, _id, _label, _rootUri, inputBoxDocumentUri);
+		this.#proxy.$registerSourceControl(this.handle, _parent?.handle, _id, _label, _rootUri, getHistoryItemIconDto(_iconPath), _isHidden, inputBoxDocumentUri);
+
+		this.onDidDisposeParent = _parent ? _parent.onDidDispose : Event.None;
 	}
 
 	private createdResourceGroups = new Map<ExtHostSourceControlResourceGroup, IDisposable>();
@@ -840,20 +913,39 @@ class ExtHostSourceControl implements vscode.SourceControl {
 		this._onDidChangeSelection.fire(selected);
 	}
 
+	async provideArtifacts(group: string, token: CancellationToken): Promise<SCMArtifactDto[] | undefined> {
+		const commandsDisposables = new DisposableStore();
+		const artifacts = await this.artifactProvider?.provideArtifacts(group, token);
+		const artifactsDto = artifacts?.map(artifact => ({
+			...artifact,
+			icon: getHistoryItemIconDto(artifact.icon),
+			command: artifact.command ? this._commands.converter.toInternal(artifact.command, commandsDisposables) : undefined
+		}));
+
+		this._artifactCommandsDisposables.get(group)?.dispose();
+		this._artifactCommandsDisposables.set(group, commandsDisposables);
+
+		return artifactsDto;
+	}
+
 	dispose(): void {
 		this._acceptInputDisposables.dispose();
 		this._actionButtonDisposables.dispose();
 		this._statusBarDisposables.dispose();
+		this._historyProviderDisposable.dispose();
+		this._artifactProviderDisposable.dispose();
+		this._artifactCommandsDisposables.dispose();
 
 		this._groups.forEach(group => group.dispose());
 		this.#proxy.$unregisterSourceControl(this.handle);
+
+		this._onDidDispose.fire();
+		this._onDidDispose.dispose();
 	}
 }
 
 export class ExtHostSCM implements ExtHostSCMShape {
 
-	private static _handlePool: number = 0;
-
 	private _proxy: MainThreadSCMShape;
 	private readonly _telemetry: MainThreadTelemetryShape;
 	private _sourceControls: Map<ProviderHandle, ExtHostSourceControl> = new Map<ProviderHandle, ExtHostSourceControl>();
@@ -912,7 +1004,7 @@ export class ExtHostSCM implements ExtHostSCMShape {
 		});
 	}
 
-	createSourceControl(extension: IExtensionDescription, id: string, label: string, rootUri: vscode.Uri | undefined): vscode.SourceControl {
+	createSourceControl(extension: IExtensionDescription, id: string, label: string, rootUri: vscode.Uri | undefined, iconPath: vscode.IconPath | undefined, isHidden: boolean | undefined, parent: vscode.SourceControl | undefined): vscode.SourceControl {
 		this.logService.trace('ExtHostSCM#createSourceControl', extension.identifier.value, id, label, rootUri);
 
 		type TEvent = { extensionId: string };
@@ -925,9 +1017,9 @@ export class ExtHostSCM implements ExtHostSCMShape {
 			extensionId: extension.identifier.value,
 		});
 
-		const handle = ExtHostSCM._handlePool++;
-		const sourceControl = new ExtHostSourceControl(extension, this._extHostDocuments, this._proxy, this._commands, id, label, rootUri);
-		this._sourceControls.set(handle, sourceControl);
+		const parentSourceControl = parent ? Iterable.find(this._sourceControls.values(), s => s === parent) : undefined;
+		const sourceControl = new ExtHostSourceControl(extension, this._extHostDocuments, this._proxy, this._commands, id, label, rootUri, iconPath, isHidden, parentSourceControl);
+		this._sourceControls.set(sourceControl.handle, sourceControl);
 
 		const sourceControls = this._sourceControlsByExtension.get(extension.identifier) || [];
 		sourceControls.push(sourceControl);
@@ -1033,6 +1125,9 @@ export class ExtHostSCM implements ExtHostSCMShape {
 
 	$setSelectedSourceControl(selectedSourceControlHandle: number | undefined): Promise<void> {
 		this.logService.trace('ExtHostSCM#$setSelectedSourceControl', selectedSourceControlHandle);
+		if (this._selectedSourceControlHandle === selectedSourceControlHandle) {
+			return Promise.resolve(undefined);
+		}
 
 		if (selectedSourceControlHandle !== undefined) {
 			this._sourceControls.get(selectedSourceControlHandle)?.setSelectionState(true);
@@ -1046,6 +1141,19 @@ export class ExtHostSCM implements ExtHostSCMShape {
 		return Promise.resolve(undefined);
 	}
 
+	async $resolveHistoryItem(sourceControlHandle: number, historyItemId: string, token: CancellationToken): Promise<SCMHistoryItemDto | undefined> {
+		try {
+			const historyProvider = this._sourceControls.get(sourceControlHandle)?.historyProvider;
+			const historyItem = await historyProvider?.resolveHistoryItem(historyItemId, token);
+
+			return historyItem ? toSCMHistoryItemDto(historyItem) : undefined;
+		}
+		catch (err) {
+			this.logService.error('ExtHostSCM#$resolveHistoryItem', err);
+			return undefined;
+		}
+	}
+
 	async $resolveHistoryItemChatContext(sourceControlHandle: number, historyItemId: string, token: CancellationToken): Promise<string | undefined> {
 		try {
 			const historyProvider = this._sourceControls.get(sourceControlHandle)?.historyProvider;
@@ -1059,6 +1167,19 @@ export class ExtHostSCM implements ExtHostSCMShape {
 		}
 	}
 
+	async $resolveHistoryItemChangeRangeChatContext(sourceControlHandle: number, historyItemId: string, historyItemParentId: string, path: string, token: CancellationToken): Promise<string | undefined> {
+		try {
+			const historyProvider = this._sourceControls.get(sourceControlHandle)?.historyProvider;
+			const chatContext = await historyProvider?.resolveHistoryItemChangeRangeChatContext?.(historyItemId, historyItemParentId, path, token);
+
+			return chatContext ?? undefined;
+		}
+		catch (err) {
+			this.logService.error('ExtHostSCM#$resolveHistoryItemChangeRangeChatContext', err);
+			return undefined;
+		}
+	}
+
 	async $resolveHistoryItemRefsCommonAncestor(sourceControlHandle: number, historyItemRefs: string[], token: CancellationToken): Promise<string | undefined> {
 		try {
 			const historyProvider = this._sourceControls.get(sourceControlHandle)?.historyProvider;
@@ -1085,7 +1206,7 @@ export class ExtHostSCM implements ExtHostSCMShape {
 		}
 	}
 
-	async $provideHistoryItems(sourceControlHandle: number, options: any, token: CancellationToken): Promise<SCMHistoryItemDto[] | undefined> {
+	async $provideHistoryItems(sourceControlHandle: number, options: vscode.SourceControlHistoryOptions, token: CancellationToken): Promise<SCMHistoryItemDto[] | undefined> {
 		try {
 			const historyProvider = this._sourceControls.get(sourceControlHandle)?.historyProvider;
 			const historyItems = await historyProvider?.provideHistoryItems(options, token);
@@ -1110,4 +1231,31 @@ export class ExtHostSCM implements ExtHostSCMShape {
 			return undefined;
 		}
 	}
+
+	async $provideArtifactGroups(sourceControlHandle: number, token: CancellationToken): Promise<SCMArtifactGroupDto[] | undefined> {
+		try {
+			const artifactProvider = this._sourceControls.get(sourceControlHandle)?.artifactProvider;
+			const groups = await artifactProvider?.provideArtifactGroups(token);
+
+			return groups?.map(group => ({
+				...group,
+				icon: getHistoryItemIconDto(group.icon)
+			}));
+		}
+		catch (err) {
+			this.logService.error('ExtHostSCM#$provideArtifactGroups', err);
+			return undefined;
+		}
+	}
+
+	async $provideArtifacts(sourceControlHandle: number, group: string, token: CancellationToken): Promise<SCMArtifactDto[] | undefined> {
+		try {
+			const sourceControl = this._sourceControls.get(sourceControlHandle);
+			return sourceControl?.provideArtifacts(group, token);
+		}
+		catch (err) {
+			this.logService.error('ExtHostSCM#$provideArtifacts', err);
+			return undefined;
+		}
+	}
 }
