@@ -4,13 +4,12 @@
  *--------------------------------------------------------------------------------------------*/
 
 import assert from 'assert';
-import { DisposableStore } from '../../../../base/common/lifecycle.js';
 import { IDialogService } from '../../../../platform/dialogs/common/dialogs.js';
 import { TestDialogService } from '../../../../platform/dialogs/test/common/testDialogService.js';
 import { TestInstantiationService } from '../../../../platform/instantiation/test/common/instantiationServiceMock.js';
 import { INotificationService } from '../../../../platform/notification/common/notification.js';
 import { TestNotificationService } from '../../../../platform/notification/test/common/testNotificationService.js';
-import { IQuickInputHideEvent, IQuickInputService, IQuickPickDidAcceptEvent } from '../../../../platform/quickinput/common/quickInput.js';
+import { IQuickInputHideEvent, IQuickInputService, IQuickPickDidAcceptEvent, IQuickPickItem, QuickInputHideReason } from '../../../../platform/quickinput/common/quickInput.js';
 import { IStorageService } from '../../../../platform/storage/common/storage.js';
 import { ITelemetryService } from '../../../../platform/telemetry/common/telemetry.js';
 import { NullTelemetryService } from '../../../../platform/telemetry/common/telemetryUtils.js';
@@ -29,7 +28,7 @@ import type { AuthenticationProvider, AuthenticationSession } from 'vscode';
 import { IBrowserWorkbenchEnvironmentService } from '../../../services/environment/browser/environmentService.js';
 import { IProductService } from '../../../../platform/product/common/productService.js';
 import { AuthenticationAccessService, IAuthenticationAccessService } from '../../../services/authentication/browser/authenticationAccessService.js';
-import { AuthenticationUsageService, IAuthenticationUsageService } from '../../../services/authentication/browser/authenticationUsageService.js';
+import { IAccountUsage, IAuthenticationUsageService } from '../../../services/authentication/browser/authenticationUsageService.js';
 import { AuthenticationExtensionsService } from '../../../services/authentication/browser/authenticationExtensionsService.js';
 import { ILogService, NullLogService } from '../../../../platform/log/common/log.js';
 import { IExtHostInitDataService } from '../../common/extHostInitDataService.js';
@@ -44,35 +43,49 @@ import { TestSecretStorageService } from '../../../../platform/secrets/test/comm
 import { IDynamicAuthenticationProviderStorageService } from '../../../services/authentication/common/dynamicAuthenticationProviderStorage.js';
 import { DynamicAuthenticationProviderStorageService } from '../../../services/authentication/browser/dynamicAuthenticationProviderStorageService.js';
 import { ExtHostProgress } from '../../common/extHostProgress.js';
+import { ensureNoDisposablesAreLeakedInTestSuite } from '../../../../base/test/common/utils.js';
+import { ServiceCollection } from '../../../../platform/instantiation/common/serviceCollection.js';
+import { SyncDescriptor } from '../../../../platform/instantiation/common/descriptors.js';
 
 class AuthQuickPick {
-	private listener: ((e: IQuickPickDidAcceptEvent) => any) | undefined;
+	private accept: ((e: IQuickPickDidAcceptEvent) => any) | undefined;
+	private hide: ((e: IQuickInputHideEvent) => any) | undefined;
 	public items = [];
-	public get selectedItems(): string[] {
+	public get selectedItems(): IQuickPickItem[] {
 		return this.items;
 	}
 
 	onDidAccept(listener: (e: IQuickPickDidAcceptEvent) => any) {
-		this.listener = listener;
+		this.accept = listener;
 	}
 	onDidHide(listener: (e: IQuickInputHideEvent) => any) {
-
+		this.hide = listener;
 	}
+
 	dispose() {
 
 	}
 	show() {
-		this.listener!({
-			inBackground: false
-		});
+		this.accept?.({ inBackground: false });
+		this.hide?.({ reason: QuickInputHideReason.Other });
 	}
 }
 class AuthTestQuickInputService extends TestQuickInputService {
 	override createQuickPick() {
+		// eslint-disable-next-line local/code-no-any-casts
 		return <any>new AuthQuickPick();
 	}
 }
 
+class TestAuthUsageService implements IAuthenticationUsageService {
+	_serviceBrand: undefined;
+	initializeExtensionUsageCache(): Promise<void> { return Promise.resolve(); }
+	extensionUsesAuth(extensionId: string): Promise<boolean> { return Promise.resolve(false); }
+	readAccountUsages(providerId: string, accountName: string): IAccountUsage[] { return []; }
+	removeAccountUsage(providerId: string, accountName: string): void { }
+	addAccountUsage(providerId: string, accountName: string, scopes: ReadonlyArray<string>, extensionId: string, extensionName: string): void { }
+}
+
 class TestAuthProvider implements AuthenticationProvider {
 	private id = 1;
 	private sessions = new Map<string, AuthenticationSession>();
@@ -111,39 +124,44 @@ class TestAuthProvider implements AuthenticationProvider {
 }
 
 suite('ExtHostAuthentication', () => {
-	let disposables: DisposableStore;
+	const disposables = ensureNoDisposablesAreLeakedInTestSuite();
 
 	let extHostAuthentication: ExtHostAuthentication;
-	let instantiationService: TestInstantiationService;
+	let mainInstantiationService: TestInstantiationService;
 
-	suiteSetup(async () => {
-		instantiationService = new TestInstantiationService();
-		instantiationService.stub(ILogService, new NullLogService());
-		instantiationService.stub(IDialogService, new TestDialogService({ confirmed: true }));
-		instantiationService.stub(IStorageService, new TestStorageService());
-		instantiationService.stub(ISecretStorageService, new TestSecretStorageService());
-		instantiationService.stub(IDynamicAuthenticationProviderStorageService, instantiationService.createInstance(DynamicAuthenticationProviderStorageService));
-		instantiationService.stub(IQuickInputService, new AuthTestQuickInputService());
-		instantiationService.stub(IExtensionService, new TestExtensionService());
+	setup(async () => {
+		// services
+		const services = new ServiceCollection();
+		services.set(ILogService, new SyncDescriptor(NullLogService));
+		services.set(IDialogService, new SyncDescriptor(TestDialogService, [{ confirmed: true }]));
+		services.set(IStorageService, new SyncDescriptor(TestStorageService));
+		services.set(ISecretStorageService, new SyncDescriptor(TestSecretStorageService));
+		services.set(IDynamicAuthenticationProviderStorageService, new SyncDescriptor(DynamicAuthenticationProviderStorageService));
+		services.set(IQuickInputService, new SyncDescriptor(AuthTestQuickInputService));
+		services.set(IExtensionService, new SyncDescriptor(TestExtensionService));
+		services.set(IActivityService, new SyncDescriptor(TestActivityService));
+		services.set(IRemoteAgentService, new SyncDescriptor(TestRemoteAgentService));
+		services.set(INotificationService, new SyncDescriptor(TestNotificationService));
+		services.set(IHostService, new SyncDescriptor(TestHostService));
+		services.set(IUserActivityService, new SyncDescriptor(UserActivityService));
+		services.set(IAuthenticationAccessService, new SyncDescriptor(AuthenticationAccessService));
+		services.set(IAuthenticationService, new SyncDescriptor(AuthenticationService));
+		services.set(IAuthenticationUsageService, new SyncDescriptor(TestAuthUsageService));
+		services.set(IAuthenticationExtensionsService, new SyncDescriptor(AuthenticationExtensionsService));
+		mainInstantiationService = disposables.add(new TestInstantiationService(services, undefined, undefined, true));
 
-		instantiationService.stub(IActivityService, new TestActivityService());
-		instantiationService.stub(IRemoteAgentService, new TestRemoteAgentService());
-		instantiationService.stub(INotificationService, new TestNotificationService());
-		instantiationService.stub(IHostService, new TestHostService());
+		// stubs
 		// eslint-disable-next-line local/code-no-dangerous-type-assertions
-		instantiationService.stub(IOpenerService, {} as Partial<IOpenerService>);
-		instantiationService.stub(IUserActivityService, new UserActivityService(instantiationService));
-		instantiationService.stub(ITelemetryService, NullTelemetryService);
-		instantiationService.stub(IBrowserWorkbenchEnvironmentService, TestEnvironmentService);
-		instantiationService.stub(IProductService, TestProductService);
-		instantiationService.stub(IAuthenticationAccessService, instantiationService.createInstance(AuthenticationAccessService));
-		instantiationService.stub(IAuthenticationService, instantiationService.createInstance(AuthenticationService));
-		instantiationService.stub(IAuthenticationUsageService, instantiationService.createInstance(AuthenticationUsageService));
-		const rpcProtocol = new TestRPCProtocol();
+		mainInstantiationService.stub(IOpenerService, {} as Partial<IOpenerService>);
+		mainInstantiationService.stub(ITelemetryService, NullTelemetryService);
+		mainInstantiationService.stub(IBrowserWorkbenchEnvironmentService, TestEnvironmentService);
+		mainInstantiationService.stub(IProductService, TestProductService);
 
-		instantiationService.stub(IAuthenticationExtensionsService, instantiationService.createInstance(AuthenticationExtensionsService));
-		rpcProtocol.set(MainContext.MainThreadAuthentication, instantiationService.createInstance(MainThreadAuthentication, rpcProtocol));
-		rpcProtocol.set(MainContext.MainThreadWindow, instantiationService.createInstance(MainThreadWindow, rpcProtocol));
+		const rpcProtocol = disposables.add(new TestRPCProtocol());
+
+		rpcProtocol.set(MainContext.MainThreadAuthentication, disposables.add(mainInstantiationService.createInstance(MainThreadAuthentication, rpcProtocol)));
+		rpcProtocol.set(MainContext.MainThreadWindow, disposables.add(mainInstantiationService.createInstance(MainThreadWindow, rpcProtocol)));
+		// eslint-disable-next-line local/code-no-any-casts
 		const initData: IExtHostInitDataService = {
 			environment: {
 				appUriScheme: 'test',
@@ -152,6 +170,7 @@ suite('ExtHostAuthentication', () => {
 		} as any;
 		extHostAuthentication = new ExtHostAuthentication(
 			rpcProtocol,
+			// eslint-disable-next-line local/code-no-any-casts
 			{
 				environment: {
 					appUriScheme: 'test',
@@ -161,13 +180,10 @@ suite('ExtHostAuthentication', () => {
 			new ExtHostWindow(initData, rpcProtocol),
 			new ExtHostUrls(rpcProtocol),
 			new ExtHostProgress(rpcProtocol),
-			new TestLoggerService(),
+			disposables.add(new TestLoggerService()),
+			new NullLogService()
 		);
 		rpcProtocol.set(ExtHostContext.ExtHostAuthentication, extHostAuthentication);
-	});
-
-	setup(async () => {
-		disposables = new DisposableStore();
 		disposables.add(extHostAuthentication.registerAuthenticationProvider('test', 'test provider', new TestAuthProvider('test')));
 		disposables.add(extHostAuthentication.registerAuthenticationProvider(
 			'test-multiple',
@@ -176,14 +192,6 @@ suite('ExtHostAuthentication', () => {
 			{ supportsMultipleAccounts: true }));
 	});
 
-	suiteTeardown(() => {
-		instantiationService.dispose();
-	});
-
-	teardown(() => {
-		disposables.dispose();
-	});
-
 	test('createIfNone - true', async () => {
 		const scopes = ['foo'];
 		const session = await extHostAuthentication.getSession(
@@ -553,5 +561,196 @@ suite('ExtHostAuthentication', () => {
 	});
 
 
+	//#endregion
+
+	//#region Race Condition and Sequencing Tests
+
+	test('concurrent operations on same provider are serialized', async () => {
+		const provider = new TestAuthProvider('concurrent-test');
+		const operationOrder: string[] = [];
+
+		// Mock the provider methods to track operation order
+		const originalCreateSession = provider.createSession.bind(provider);
+		const originalGetSessions = provider.getSessions.bind(provider);
+
+		provider.createSession = async (scopes) => {
+			operationOrder.push(`create-start-${scopes[0]}`);
+			await new Promise(resolve => setTimeout(resolve, 20)); // Simulate async work
+			const result = await originalCreateSession(scopes);
+			operationOrder.push(`create-end-${scopes[0]}`);
+			return result;
+		};
+
+		provider.getSessions = async (scopes) => {
+			const scopeKey = scopes ? scopes[0] : 'all';
+			operationOrder.push(`get-start-${scopeKey}`);
+			await new Promise(resolve => setTimeout(resolve, 10)); // Simulate async work
+			const result = await originalGetSessions(scopes);
+			operationOrder.push(`get-end-${scopeKey}`);
+			return result;
+		};
+
+		const disposable = extHostAuthentication.registerAuthenticationProvider('concurrent-test', 'Concurrent Test', provider);
+		disposables.add(disposable);
+
+		// Start multiple operations simultaneously on the same provider
+		const promises = [
+			extHostAuthentication.getSession(extensionDescription, 'concurrent-test', ['scope1'], { createIfNone: true }),
+			extHostAuthentication.getSession(extensionDescription, 'concurrent-test', ['scope2'], { createIfNone: true }),
+			extHostAuthentication.getSession(extensionDescription, 'concurrent-test', ['scope1'], {}) // This should get the existing session
+		];
+
+		await Promise.all(promises);
+
+		// Verify that operations were serialized - no overlapping operations
+		// Build a map of operation starts to their corresponding ends
+		const operationPairs: Array<{ start: number; end: number; operation: string }> = [];
+
+		for (let i = 0; i < operationOrder.length; i++) {
+			const current = operationOrder[i];
+			if (current.includes('-start-')) {
+				const scope = current.split('-start-')[1];
+				const operationType = current.split('-start-')[0];
+				const endOperation = `${operationType}-end-${scope}`;
+				const endIndex = operationOrder.indexOf(endOperation, i + 1);
+
+				if (endIndex !== -1) {
+					operationPairs.push({
+						start: i,
+						end: endIndex,
+						operation: `${operationType}-${scope}`
+					});
+				}
+			}
+		}
+
+		// Verify no operations overlap (serialization)
+		for (let i = 0; i < operationPairs.length; i++) {
+			for (let j = i + 1; j < operationPairs.length; j++) {
+				const op1 = operationPairs[i];
+				const op2 = operationPairs[j];
+
+				// Operations should not overlap - one should completely finish before the other starts
+				const op1EndsBeforeOp2Starts = op1.end < op2.start;
+				const op2EndsBeforeOp1Starts = op2.end < op1.start;
+
+				assert.ok(op1EndsBeforeOp2Starts || op2EndsBeforeOp1Starts,
+					`Operations ${op1.operation} and ${op2.operation} should not overlap. ` +
+					`Op1: ${op1.start}-${op1.end}, Op2: ${op2.start}-${op2.end}. ` +
+					`Order: [${operationOrder.join(', ')}]`);
+			}
+		}
+
+		// Verify we have the expected operations
+		assert.ok(operationOrder.includes('create-start-scope1'), 'Should have created session for scope1');
+		assert.ok(operationOrder.includes('create-end-scope1'), 'Should have completed creating session for scope1');
+		assert.ok(operationOrder.includes('create-start-scope2'), 'Should have created session for scope2');
+		assert.ok(operationOrder.includes('create-end-scope2'), 'Should have completed creating session for scope2');
+
+		// The third call should use getSessions to find the existing scope1 session
+		assert.ok(operationOrder.includes('get-start-scope1'), 'Should have called getSessions for existing scope1 session');
+		assert.ok(operationOrder.includes('get-end-scope1'), 'Should have completed getSessions for existing scope1 session');
+	});
+
+	test('provider registration and immediate disposal race condition', async () => {
+		const provider = new TestAuthProvider('race-test');
+
+		// Register and immediately dispose
+		const disposable = extHostAuthentication.registerAuthenticationProvider('race-test', 'Race Test', provider);
+		disposable.dispose();
+
+		// Try to use the provider after disposal - should fail gracefully
+		try {
+			await extHostAuthentication.getSession(extensionDescription, 'race-test', ['scope'], { createIfNone: true });
+			assert.fail('Should have thrown an error for non-existent provider');
+		} catch (error) {
+			// Expected - provider should be unavailable
+			assert.ok(error);
+		}
+	});
+
+	test('provider re-registration after proper disposal', async () => {
+		const provider1 = new TestAuthProvider('reregister-test-1');
+		const provider2 = new TestAuthProvider('reregister-test-2');
+
+		// First registration
+		const disposable1 = extHostAuthentication.registerAuthenticationProvider('reregister-test', 'Provider 1', provider1);
+
+		// Create a session with first provider
+		const session1 = await extHostAuthentication.getSession(extensionDescription, 'reregister-test', ['scope'], { createIfNone: true });
+		assert.strictEqual(session1?.account.label, 'reregister-test-1');
+
+		// Dispose first provider
+		disposable1.dispose();
+
+		// Re-register with different provider
+		const disposable2 = extHostAuthentication.registerAuthenticationProvider('reregister-test', 'Provider 2', provider2);
+		disposables.add(disposable2);
+
+		// Create session with second provider
+		const session2 = await extHostAuthentication.getSession(extensionDescription, 'reregister-test', ['scope'], { createIfNone: true });
+		assert.strictEqual(session2?.account.label, 'reregister-test-2');
+		assert.notStrictEqual(session1?.accessToken, session2?.accessToken);
+	});
+
+	test('operations on different providers run concurrently', async () => {
+		const provider1 = new TestAuthProvider('concurrent-1');
+		const provider2 = new TestAuthProvider('concurrent-2');
+
+		let provider1Started = false;
+		let provider2Started = false;
+		let provider1Finished = false;
+		let provider2Finished = false;
+		let concurrencyVerified = false;
+
+		// Override createSession to track timing
+		const originalCreate1 = provider1.createSession.bind(provider1);
+		const originalCreate2 = provider2.createSession.bind(provider2);
+
+		provider1.createSession = async (scopes) => {
+			provider1Started = true;
+			await new Promise(resolve => setTimeout(resolve, 20));
+			const result = await originalCreate1(scopes);
+			provider1Finished = true;
+			return result;
+		};
+
+		provider2.createSession = async (scopes) => {
+			provider2Started = true;
+			// Provider 2 should start before provider 1 finishes (concurrent execution)
+			if (provider1Started && !provider1Finished) {
+				concurrencyVerified = true;
+			}
+			await new Promise(resolve => setTimeout(resolve, 10));
+			const result = await originalCreate2(scopes);
+			provider2Finished = true;
+			return result;
+		};
+
+		const disposable1 = extHostAuthentication.registerAuthenticationProvider('concurrent-1', 'Concurrent 1', provider1);
+		const disposable2 = extHostAuthentication.registerAuthenticationProvider('concurrent-2', 'Concurrent 2', provider2);
+		disposables.add(disposable1);
+		disposables.add(disposable2);
+
+		// Start operations on both providers simultaneously
+		const [session1, session2] = await Promise.all([
+			extHostAuthentication.getSession(extensionDescription, 'concurrent-1', ['scope'], { createIfNone: true }),
+			extHostAuthentication.getSession(extensionDescription, 'concurrent-2', ['scope'], { createIfNone: true })
+		]);
+
+		// Verify both operations completed successfully
+		assert.ok(session1);
+		assert.ok(session2);
+		assert.ok(provider1Started, 'Provider 1 should have started');
+		assert.ok(provider2Started, 'Provider 2 should have started');
+		assert.ok(provider1Finished, 'Provider 1 should have finished');
+		assert.ok(provider2Finished, 'Provider 2 should have finished');
+		assert.strictEqual(session1.account.label, 'concurrent-1');
+		assert.strictEqual(session2.account.label, 'concurrent-2');
+
+		// Verify that operations ran concurrently (provider 2 started while provider 1 was still running)
+		assert.ok(concurrencyVerified, 'Operations should have run concurrently - provider 2 should start while provider 1 is still running');
+	});
+
 	//#endregion
 });
