@@ -7,14 +7,13 @@ import { DeferredPromise, disposableTimeout, RunOnceScheduler } from '../../../.
 import { CancellationToken, CancellationTokenSource } from '../../../../base/common/cancellation.js';
 import { Codicon } from '../../../../base/common/codicons.js';
 import { Event } from '../../../../base/common/event.js';
-import { DisposableStore, IDisposable, toDisposable } from '../../../../base/common/lifecycle.js';
-import { autorun, derived } from '../../../../base/common/observable.js';
+import { DisposableStore, IDisposable, toDisposable, Disposable } from '../../../../base/common/lifecycle.js';
+import { autorun, derived, observableValue, IObservable } from '../../../../base/common/observable.js';
 import { ThemeIcon } from '../../../../base/common/themables.js';
 import { URI } from '../../../../base/common/uri.js';
 import { generateUuid } from '../../../../base/common/uuid.js';
 import { localize } from '../../../../nls.js';
-import { IDialogService } from '../../../../platform/dialogs/common/dialogs.js';
-import { ByteSize, IFileService } from '../../../../platform/files/common/files.js';
+import { ByteSize, IFileService, IFileStat } from '../../../../platform/files/common/files.js';
 import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
 import { INotificationService } from '../../../../platform/notification/common/notification.js';
 import { DefaultQuickAccessFilterValue, IQuickAccessProvider, IQuickAccessProviderRunOptions } from '../../../../platform/quickinput/common/quickAccess.js';
@@ -22,13 +21,20 @@ import { IQuickInputService, IQuickPick, IQuickPickItem, IQuickPickSeparator } f
 import { IEditorService } from '../../../services/editor/common/editorService.js';
 import { IViewsService } from '../../../services/views/common/viewsService.js';
 import { IChatWidgetService } from '../../chat/browser/chat.js';
-import { resolveImageEditorAttachContext } from '../../chat/browser/chatAttachmentResolve.js';
-import { IChatRequestVariableEntry } from '../../chat/common/chatModel.js';
+import { IChatAttachmentResolveService } from '../../chat/browser/attachments/chatAttachmentResolveService.js';
+import { IChatRequestVariableEntry } from '../../chat/common/attachments/chatVariableEntries.js';
 import { IMcpResource, IMcpResourceTemplate, IMcpServer, IMcpService, isMcpResourceTemplate, McpCapability, McpConnectionState, McpResourceURI } from '../common/mcpTypes.js';
+import { McpIcons } from '../common/mcpIcons.js';
 import { IUriTemplateVariable } from '../common/uriTemplate.js';
-import { openPanelChatAndGetWidget } from './mcpCommands.js';
+import { openPanelChatAndGetWidget } from './openPanelChatAndGetWidget.js';
+import { LinkedList } from '../../../../base/common/linkedList.js';
+import { ChatContextPickAttachment } from '../../chat/browser/attachments/chatContextPickService.js';
+import { asArray } from '../../../../base/common/arrays.js';
 
-export class McpResourcePickHelper {
+export class McpResourcePickHelper extends Disposable {
+	private _resources = observableValue<{ picks: Map<IMcpServer, (IMcpResourceTemplate | IMcpResource)[]>; isBusy: boolean }>(this, { picks: new Map(), isBusy: true });
+	private _pickItemsStack: LinkedList<{ server: IMcpServer; resources: (IMcpResource | IMcpResourceTemplate)[] }> = new LinkedList();
+	private _inDirectory = observableValue<undefined | { server: IMcpServer; resources: (IMcpResource | IMcpResourceTemplate)[] }>(this, undefined);
 	public static sep(server: IMcpServer): IQuickPickSeparator {
 		return {
 			id: server.definition.id,
@@ -37,28 +43,58 @@ export class McpResourcePickHelper {
 		};
 	}
 
+	public addCurrentMCPQuickPickItemLevel(server: IMcpServer, resources: (IMcpResource | IMcpResourceTemplate)[]): void {
+		let isValidPush: boolean = false;
+		isValidPush = this._pickItemsStack.isEmpty();
+		if (!isValidPush) {
+			const stackedItem = this._pickItemsStack.peek();
+			if (stackedItem?.server === server && stackedItem.resources === resources) {
+				isValidPush = false;
+			} else {
+				isValidPush = true;
+			}
+		}
+		if (isValidPush) {
+			this._pickItemsStack.push({ server, resources });
+		}
+
+	}
+
+	public navigateBack(): boolean {
+		const items = this._pickItemsStack.pop();
+		if (items) {
+			this._inDirectory.set({ server: items.server, resources: items.resources }, undefined);
+			return true;
+		} else {
+			return false;
+		}
+	}
+
 	public static item(resource: IMcpResource | IMcpResourceTemplate): IQuickPickItem {
+		const iconPath = resource.icons.getUrl(22);
 		if (isMcpResourceTemplate(resource)) {
 			return {
 				id: resource.template.template,
-				label: resource.name,
+				label: resource.title || resource.name,
 				description: resource.description,
 				detail: localize('mcp.resource.template', 'Resource template: {0}', resource.template.template),
+				iconPath,
 			};
 		}
 
 		return {
 			id: resource.uri.toString(),
-			label: resource.name,
+			label: resource.title || resource.name,
 			description: resource.description,
 			detail: resource.mcpUri + (resource.sizeInBytes !== undefined ? ' (' + ByteSize.formatSize(resource.sizeInBytes) + ')' : ''),
+			iconPath,
 		};
 	}
 
 	public hasServersWithResources = derived(reader => {
 		let enabled = false;
 		for (const server of this._mcpService.servers.read(reader)) {
-			const cap = server.capabilities.get();
+			const cap = server.capabilities.read(undefined);
 			if (cap === undefined) {
 				enabled = true; // until we know more
 			} else if (cap & McpCapability.Resources) {
@@ -75,16 +111,78 @@ export class McpResourcePickHelper {
 	constructor(
 		@IMcpService private readonly _mcpService: IMcpService,
 		@IFileService private readonly _fileService: IFileService,
-		@IDialogService private readonly _dialogService: IDialogService,
 		@IQuickInputService private readonly _quickInputService: IQuickInputService,
 		@INotificationService private readonly _notificationService: INotificationService,
-	) { }
+		@IChatAttachmentResolveService private readonly _chatAttachmentResolveService: IChatAttachmentResolveService
+	) {
+		super();
+	}
 
-	public async toAttachment(resource: IMcpResource | IMcpResourceTemplate): Promise<IChatRequestVariableEntry | undefined> {
+	/**
+	 * Navigate to a resource if it's a directory.
+	 * Returns true if the resource is a directory with children (navigation succeeded).
+	 * Returns false if the resource is a leaf file (no navigation).
+	 * When returning true, statefully updates the picker state to display directory contents.
+	 */
+	public async navigate(resource: IMcpResource | IMcpResourceTemplate, server: IMcpServer): Promise<boolean> {
 		if (isMcpResourceTemplate(resource)) {
-			return this._resourceTemplateToAttachment(resource);
+			return false;
+		}
+
+		const uri = resource.uri;
+		let stat: IFileStat | undefined = undefined;
+		try {
+			stat = await this._fileService.resolve(uri, { resolveMetadata: false });
+		} catch (e) {
+			return false;
+		}
+
+		if (stat && this._isDirectoryResource(resource) && (stat.children?.length ?? 0) > 0) {
+			// Save current state to stack before navigating
+			const currentResources = this._resources.get().picks.get(server);
+			if (currentResources) {
+				this.addCurrentMCPQuickPickItemLevel(server, currentResources);
+			}
+
+			// Convert all the children to IMcpResource objects
+			const childResources: IMcpResource[] = stat.children!.map(child => {
+				const mcpUri = McpResourceURI.fromServer(server.definition, child.resource.toString());
+				return {
+					uri: mcpUri,
+					mcpUri: child.resource.path,
+					name: child.name,
+					title: child.name,
+					description: resource.description,
+					mimeType: undefined,
+					sizeInBytes: child.size,
+					icons: McpIcons.fromParsed(undefined)
+				};
+			});
+			this._inDirectory.set({ server, resources: childResources }, undefined);
+			return true;
+		}
+		return false;
+	}
+
+	public toAttachment(resource: IMcpResource | IMcpResourceTemplate, server: IMcpServer): Promise<ChatContextPickAttachment> | 'noop' {
+		const noop = 'noop';
+		if (this._isDirectoryResource(resource)) {
+			//Check if directory
+			this.checkIfDirectoryAndPopulate(resource, server);
+			return noop;
+		}
+		if (isMcpResourceTemplate(resource)) {
+			return this._resourceTemplateToAttachment(resource).then(val => val || noop);
 		} else {
-			return this._resourceToAttachment(resource);
+			return this._resourceToAttachment(resource).then(val => val || noop);
+		}
+	}
+
+	public async checkIfDirectoryAndPopulate(resource: IMcpResource | IMcpResourceTemplate, server: IMcpServer): Promise<boolean> {
+		try {
+			return !await this.navigate(resource, server);
+		} catch (error) {
+			return false;
 		}
 	}
 
@@ -97,8 +195,10 @@ export class McpResourcePickHelper {
 		}
 	}
 
+	public checkIfNestedResources = () => !this._pickItemsStack.isEmpty();
+
 	private async _resourceToAttachment(resource: { uri: URI; name: string; mimeType?: string }): Promise<IChatRequestVariableEntry | undefined> {
-		const asImage = await resolveImageEditorAttachContext(this._fileService, this._dialogService, resource.uri, undefined, resource.mimeType);
+		const asImage = await this._chatAttachmentResolveService.resolveImageEditorAttachContext(resource.uri, undefined, resource.mimeType);
 		if (asImage) {
 			return asImage;
 		}
@@ -119,6 +219,7 @@ export class McpResourcePickHelper {
 			name: rt.name,
 			mimeType: rt.mimeType,
 		});
+
 	}
 
 	private async _verifyUriIfNeeded({ uri, needsVerification }: { uri: URI; needsVerification: boolean }): Promise<URI | undefined> {
@@ -131,7 +232,7 @@ export class McpResourcePickHelper {
 			return uri;
 		}
 
-		this._notificationService.warn(localize('mcp.resource.template.notFound', "The resource {0} was not found.", McpResourceURI.toServer(uri).resourceURI.toString()));
+		this._notificationService.warn(localize('mcp.resource.template.notFound', "The resource {0} was not found.", McpResourceURI.toServer(uri).resourceURL.toString()));
 		return undefined;
 	}
 
@@ -197,7 +298,9 @@ export class McpResourcePickHelper {
 			input.items = items;
 		};
 
-		let changeCancellation = store.add(new CancellationTokenSource());
+		let changeCancellation = new CancellationTokenSource();
+		store.add(toDisposable(() => changeCancellation.dispose(true)));
+
 		const getCompletionItems = () => {
 			const inputValue = input.value;
 			let promise = completions.get(inputValue);
@@ -237,8 +340,7 @@ export class McpResourcePickHelper {
 			store.add(input.onDidChangeValue(value => {
 				input.busy = true;
 				changeCancellation.dispose(true);
-				store.delete(changeCancellation);
-				changeCancellation = store.add(new CancellationTokenSource());
+				changeCancellation = new CancellationTokenSource();
 				getCompletionItemsScheduler.cancel();
 				setItems(value);
 
@@ -253,15 +355,25 @@ export class McpResourcePickHelper {
 		}).finally(() => store.dispose());
 	}
 
-	public getPicks(onChange: (value: Map<IMcpServer, (IMcpResourceTemplate | IMcpResource)[]>) => void, token?: CancellationToken) {
-		const cts = new CancellationTokenSource(token);
-		const store = new DisposableStore();
-		store.add(toDisposable(() => cts.dispose(true)));
+	private _isDirectoryResource(resource: IMcpResource | IMcpResourceTemplate): boolean {
 
+		if (resource.mimeType && resource.mimeType === 'inode/directory') {
+			return true;
+		} else if (isMcpResourceTemplate(resource)) {
+			return resource.template.template.endsWith('/');
+		} else {
+			return resource.uri.path.endsWith('/');
+		}
+	}
+
+	public getPicks(token?: CancellationToken): IObservable<{ picks: Map<IMcpServer, (IMcpResourceTemplate | IMcpResource)[]>; isBusy: boolean }> {
+		const cts = new CancellationTokenSource(token);
+		let isBusyLoadingPicks = true;
+		this._register(toDisposable(() => cts.dispose(true)));
 		// We try to show everything in-sequence to avoid flickering (#250411) as long as
 		// it loads within 5 seconds. Otherwise we just show things as the load in parallel.
 		let showInSequence = true;
-		store.add(disposableTimeout(() => {
+		this._register(disposableTimeout(() => {
 			showInSequence = false;
 			publish();
 		}, 5_000));
@@ -282,14 +394,14 @@ export class McpResourcePickHelper {
 					break;
 				}
 			}
-			onChange(output);
+			this._resources.set({ picks: output, isBusy: isBusyLoadingPicks }, undefined);
 		};
 
 		type Rec = { templates: DeferredPromise<IMcpResourceTemplate[]>; resourcesSoFar: IMcpResource[]; resources: DeferredPromise<unknown> };
 
 		const servers = new Map<IMcpServer, Rec>();
 		// Enumerate servers and start servers that need to be started to get capabilities
-		return Promise.all((this.explicitServers || this._mcpService.servers.get()).map(async server => {
+		Promise.all((this.explicitServers || this._mcpService.servers.get()).map(async server => {
 			let cap = server.capabilities.get();
 			const rec: Rec = {
 				templates: new DeferredPromise(),
@@ -305,8 +417,8 @@ export class McpResourcePickHelper {
 							resolve(undefined);
 						}
 					});
-					store.add(cts.token.onCancellationRequested(() => resolve(undefined)));
-					store.add(autorun(reader => {
+					this._register(cts.token.onCancellationRequested(() => resolve(undefined)));
+					this._register(autorun(reader => {
 						const cap2 = server.capabilities.read(reader);
 						if (cap2 !== undefined) {
 							resolve(cap2);
@@ -329,14 +441,21 @@ export class McpResourcePickHelper {
 				rec.templates.complete([]);
 				rec.resources.complete([]);
 			}
-			publish();
 		})).finally(() => {
-			store.dispose();
+			isBusyLoadingPicks = false;
+			publish();
+		});
+
+		// Use derived to compute the appropriate resource map based on directory navigation state
+		return derived(this, reader => {
+			const directoryResource = this._inDirectory.read(reader);
+			return directoryResource
+				? { picks: new Map([[directoryResource.server, directoryResource.resources]]), isBusy: false }
+				: this._resources.read(reader);
 		});
 	}
 }
 
-
 export abstract class AbstractMcpResourceAccessPick {
 	constructor(
 		private readonly _scopeTo: IMcpServer | undefined,
@@ -344,66 +463,98 @@ export abstract class AbstractMcpResourceAccessPick {
 		@IEditorService private readonly _editorService: IEditorService,
 		@IChatWidgetService protected readonly _chatWidgetService: IChatWidgetService,
 		@IViewsService private readonly _viewsService: IViewsService,
-	) { }
+	) {
+	}
 
 	protected applyToPick(picker: IQuickPick<IQuickPickItem, { useSeparators: true }>, token: CancellationToken, runOptions?: IQuickAccessProviderRunOptions) {
 		picker.canAcceptInBackground = true;
 		picker.busy = true;
 		picker.keepScrollPosition = true;
+		const store = new DisposableStore();
+		const goBackId = '_goback_';
 
-		type ResourceQuickPickItem = IQuickPickItem & { resource: IMcpResource | IMcpResourceTemplate };
+		type ResourceQuickPickItem = IQuickPickItem & { resource: IMcpResource | IMcpResourceTemplate; server: IMcpServer };
 
 		const attachButton = localize('mcp.quickaccess.attach', "Attach to chat");
 
-		const helper = this._instantiationService.createInstance(McpResourcePickHelper);
+		const helper = store.add(this._instantiationService.createInstance(McpResourcePickHelper));
 		if (this._scopeTo) {
 			helper.explicitServers = [this._scopeTo];
 		}
-		helper.getPicks(servers => {
-			const items: (ResourceQuickPickItem | IQuickPickSeparator)[] = [];
-			for (const [server, resources] of servers) {
+		const picksObservable = helper.getPicks(token);
+		store.add(autorun(reader => {
+			const pickItems = picksObservable.read(reader);
+			const isBusy = pickItems.isBusy;
+			const items: (ResourceQuickPickItem | IQuickPickSeparator | IQuickPickItem)[] = [];
+			for (const [server, resources] of pickItems.picks) {
 				items.push(McpResourcePickHelper.sep(server));
 				for (const resource of resources) {
 					const pickItem = McpResourcePickHelper.item(resource);
 					pickItem.buttons = [{ iconClass: ThemeIcon.asClassName(Codicon.attach), tooltip: attachButton }];
-					items.push({ ...pickItem, resource });
+					items.push({ ...pickItem, resource, server });
 				}
 			}
+			if (helper.checkIfNestedResources()) {
+				// Add go back item
+				const goBackItem: IQuickPickItem = {
+					id: goBackId,
+					label: localize('goBack', 'Go back ↩'),
+					alwaysShow: true
+				};
+				items.push(goBackItem);
+			}
 			picker.items = items;
-		}, token).finally(() => {
-			picker.busy = false;
-		});
+			picker.busy = isBusy;
+		}));
 
-		const store = new DisposableStore();
 		store.add(picker.onDidTriggerItemButton(event => {
 			if (event.button.tooltip === attachButton) {
 				picker.busy = true;
-				helper.toAttachment((event.item as ResourceQuickPickItem).resource).then(async a => {
-					if (a) {
-						const widget = await openPanelChatAndGetWidget(this._viewsService, this._chatWidgetService);
-						widget?.attachmentModel.addContext(a);
-					}
-					picker.hide();
-				});
-			}
-		}));
-
-		store.add(picker.onDidAccept(async event => {
-			if (!event.inBackground) {
-				picker.hide(); // hide picker unless we accept in background
-			}
-
-			if (runOptions?.handleAccept) {
-				runOptions.handleAccept?.(picker.activeItems[0], event.inBackground);
-			} else {
-				const [item] = picker.selectedItems;
-				const uri = await helper.toURI((item as ResourceQuickPickItem).resource);
-				if (uri) {
-					this._editorService.openEditor({ resource: uri, options: { preserveFocus: event.inBackground } });
+				const resourceItem = event.item as ResourceQuickPickItem;
+				const attachment = helper.toAttachment(resourceItem.resource, resourceItem.server);
+				if (attachment instanceof Promise) {
+					attachment.then(async a => {
+						if (a !== 'noop') {
+							const widget = await openPanelChatAndGetWidget(this._viewsService, this._chatWidgetService);
+							widget?.attachmentModel.addContext(...asArray(a));
+						}
+						picker.hide();
+					});
 				}
 			}
 		}));
 
+		store.add(picker.onDidHide(() => {
+			helper.dispose();
+		}));
+
+		store.add(picker.onDidAccept(async event => {
+			try {
+				picker.busy = true;
+				const [item] = picker.selectedItems;
+
+				// Check if go back item was selected
+				if (item.id === goBackId) {
+					helper.navigateBack();
+					picker.busy = false;
+					return;
+				}
+
+				const resourceItem = item as ResourceQuickPickItem;
+				const resource = resourceItem.resource;
+				// Try to navigate into the resource if it's a directory
+				const isNested = await helper.navigate(resource, resourceItem.server);
+				if (!isNested) {
+					const uri = await helper.toURI(resource);
+					if (uri) {
+						picker.hide();
+						this._editorService.openEditor({ resource: uri, options: { preserveFocus: event.inBackground } });
+					}
+				}
+			} finally {
+				picker.busy = false;
+			}
+		}));
 		return store;
 	}
 }
@@ -440,7 +591,7 @@ export class McpResourceQuickAccess extends AbstractMcpResourceAccessPick implem
 		@IInstantiationService instantiationService: IInstantiationService,
 		@IEditorService editorService: IEditorService,
 		@IChatWidgetService chatWidgetService: IChatWidgetService,
-		@IViewsService viewsService: IViewsService,
+		@IViewsService viewsService: IViewsService
 	) {
 		super(undefined, instantiationService, editorService, chatWidgetService, viewsService);
 	}
