@@ -33,7 +33,7 @@ import { INotebookCellOutlineDataSource, NotebookCellOutlineDataSource } from '.
 import { CellKind, NotebookCellsChangeType, NotebookSetting } from '../../../common/notebookCommon.js';
 import { IEditorService, SIDE_GROUP } from '../../../../../services/editor/common/editorService.js';
 import { LifecyclePhase } from '../../../../../services/lifecycle/common/lifecycle.js';
-import { IBreadcrumbsDataSource, IOutline, IOutlineComparator, IOutlineCreator, IOutlineListConfig, IOutlineService, IQuickPickDataSource, IQuickPickOutlineElement, OutlineChangeEvent, OutlineConfigCollapseItemsValues, OutlineConfigKeys, OutlineTarget } from '../../../../../services/outline/browser/outline.js';
+import { IBreadcrumbsDataSource, IBreadcrumbsOutlineElement, IOutline, IOutlineComparator, IOutlineCreator, IOutlineListConfig, IOutlineService, IQuickPickDataSource, IQuickPickOutlineElement, OutlineChangeEvent, OutlineConfigCollapseItemsValues, OutlineConfigKeys, OutlineTarget } from '../../../../../services/outline/browser/outline.js';
 import { OutlineEntry } from '../../viewModel/OutlineEntry.js';
 import { CancellationToken } from '../../../../../../base/common/cancellation.js';
 import { IModelDeltaDecoration } from '../../../../../../editor/common/model.js';
@@ -466,12 +466,12 @@ export class NotebookBreadcrumbsProvider implements IBreadcrumbsDataSource<Outli
 		}));
 	}
 
-	getBreadcrumbElements(): readonly OutlineEntry[] {
-		const result: OutlineEntry[] = [];
+	getBreadcrumbElements(): readonly IBreadcrumbsOutlineElement<OutlineEntry>[] {
+		const result: IBreadcrumbsOutlineElement<OutlineEntry>[] = [];
 		let candidate = this.outlineDataSourceRef?.object?.activeElement;
 		while (candidate) {
 			if (this.showCodeCells || candidate.cell.cellKind !== CellKind.Code) {
-				result.unshift(candidate);
+				result.unshift({ element: candidate, label: candidate.label });
 			}
 			candidate = candidate.parent;
 		}
@@ -595,7 +595,7 @@ export class NotebookCellOutline implements IOutline<OutlineEntry> {
 			delegate,
 			renderers,
 			comparator,
-			options
+			options,
 		};
 	}
 
@@ -740,7 +740,7 @@ export class NotebookCellOutline implements IOutline<OutlineEntry> {
 		const notebookEditorOptions: INotebookEditorOptions = {
 			...options,
 			override: this._editor.input?.editorId,
-			cellRevealType: CellRevealType.NearTopIfOutsideViewport,
+			cellRevealType: CellRevealType.Top,
 			selection: entry.position,
 			viewState: undefined,
 		};
@@ -875,7 +875,7 @@ Registry.as<IConfigurationRegistry>(ConfigurationExtensions.Configuration).regis
 		[NotebookSetting.outlineShowCodeCellSymbols]: {
 			type: 'boolean',
 			default: true,
-			markdownDescription: localize('outline.showCodeCellSymbols', "When enabled, notebook outline shows code cell symbols. Relies on `notebook.outline.showCodeCells` being enabled.")
+			markdownDescription: localize('outline.showCodeCellSymbols', "When enabled, notebook outline shows code cell symbols. Relies on `#notebook.outline.showCodeCells#` being enabled.")
 		},
 		[NotebookSetting.breadcrumbsShowCodeCells]: {
 			type: 'boolean',
@@ -915,7 +915,7 @@ registerAction2(class ToggleShowMarkdownHeadersOnly extends Action2 {
 		});
 	}
 
-	run(accessor: ServicesAccessor, ...args: any[]) {
+	run(accessor: ServicesAccessor, ...args: unknown[]) {
 		const configurationService = accessor.get(IConfigurationService);
 		const showMarkdownHeadersOnly = configurationService.getValue<boolean>(NotebookSetting.outlineShowMarkdownHeadersOnly);
 		configurationService.updateValue(NotebookSetting.outlineShowMarkdownHeadersOnly, !showMarkdownHeadersOnly);
@@ -939,7 +939,7 @@ registerAction2(class ToggleCodeCellEntries extends Action2 {
 		});
 	}
 
-	run(accessor: ServicesAccessor, ...args: any[]) {
+	run(accessor: ServicesAccessor, ...args: unknown[]) {
 		const configurationService = accessor.get(IConfigurationService);
 		const showCodeCells = configurationService.getValue<boolean>(NotebookSetting.outlineShowCodeCells);
 		configurationService.updateValue(NotebookSetting.outlineShowCodeCells, !showCodeCells);
@@ -963,7 +963,7 @@ registerAction2(class ToggleCodeCellSymbolEntries extends Action2 {
 		});
 	}
 
-	run(accessor: ServicesAccessor, ...args: any[]) {
+	run(accessor: ServicesAccessor, ...args: unknown[]) {
 		const configurationService = accessor.get(IConfigurationService);
 		const showCodeCellSymbols = configurationService.getValue<boolean>(NotebookSetting.outlineShowCodeCellSymbols);
 		configurationService.updateValue(NotebookSetting.outlineShowCodeCellSymbols, !showCodeCellSymbols);
