@@ -40,6 +40,7 @@ import { Categories } from '../../../../platform/action/common/actionCommonCateg
 import { IOutputService } from '../../../services/output/common/output.js';
 import { ILoggerResource, ILoggerService, LogLevel } from '../../../../platform/log/common/log.js';
 import { VerifyExtensionSignatureConfigKey } from '../../../../platform/extensionManagement/common/extensionManagement.js';
+import { TerminalContribSettingId } from '../../terminal/terminalContribExports.js';
 
 type TelemetryData = {
 	mimeType: TelemetryTrustedValue<string>;
@@ -383,6 +384,15 @@ class ConfigurationTelemetryContribution extends Disposable implements IWorkbenc
 				}>('window.titleBarStyle', { settingValue: this.getValueToReport(key, target), source });
 				return;
 
+			case 'workbench.secondarySideBar.defaultVisibility':
+				this.telemetryService.publicLog2<UpdatedSettingEvent, {
+					owner: 'bpasero';
+					comment: 'This is used to know if secondary side bar is visible or not';
+					settingValue: { classification: 'SystemMetaData'; purpose: 'FeatureInsight'; comment: 'value of the setting' };
+					source: { classification: 'SystemMetaData'; purpose: 'FeatureInsight'; comment: 'source of the setting' };
+				}>('workbench.secondarySideBar.defaultVisibility', { settingValue: this.getValueToReport(key, target), source });
+				return;
+
 			case VerifyExtensionSignatureConfigKey:
 				this.telemetryService.publicLog2<UpdatedSettingEvent, {
 					owner: 'sandy081';
@@ -417,6 +427,23 @@ class ConfigurationTelemetryContribution extends Disposable implements IWorkbenc
 					source: { classification: 'SystemMetaData'; purpose: 'FeatureInsight'; comment: 'source of the setting' };
 				}>('extensions.autoRestart', { settingValue: this.getValueToReport(key, target), source });
 				return;
+			case TerminalContribSettingId.OutputLocation:
+				this.telemetryService.publicLog2<UpdatedSettingEvent, {
+					owner: 'meganrogge';
+					comment: 'This is used to know the output location for chat terminals';
+					settingValue: { classification: 'SystemMetaData'; purpose: 'FeatureInsight'; comment: 'value of the setting' };
+					source: { classification: 'SystemMetaData'; purpose: 'FeatureInsight'; comment: 'source of the setting' };
+				}>('terminal.integrated.chatAgentTools.outputLocation', { settingValue: this.getValueToReport(key, target), source });
+				return;
+			case TerminalContribSettingId.SuggestEnabled:
+
+				this.telemetryService.publicLog2<UpdatedSettingEvent, {
+					owner: 'meganrogge';
+					comment: 'This is used to know if terminal suggestions are enabled or not';
+					settingValue: { classification: 'SystemMetaData'; purpose: 'FeatureInsight'; comment: 'value of the setting' };
+					source: { classification: 'SystemMetaData'; purpose: 'FeatureInsight'; comment: 'source of the setting' };
+				}>('terminal.integrated.suggest.enabled', { settingValue: this.getValueToReport(key, target), source });
+				return;
 		}
 	}
 
