@@ -8,40 +8,41 @@ import { Delayer } from '../../../../../base/common/async.js';
 import { VSBuffer } from '../../../../../base/common/buffer.js';
 import { Event } from '../../../../../base/common/event.js';
 import { Iterable } from '../../../../../base/common/iterator.js';
-import { Lazy } from '../../../../../base/common/lazy.js';
 import { Disposable, DisposableStore, IDisposable, IReference, MutableDisposable, combinedDisposable, toDisposable } from '../../../../../base/common/lifecycle.js';
+import { ScrollEvent } from '../../../../../base/common/scrollable.js';
 import { URI } from '../../../../../base/common/uri.js';
 import { ICodeEditor, IDiffEditorConstructionOptions } from '../../../../../editor/browser/editorBrowser.js';
 import { CodeEditorWidget } from '../../../../../editor/browser/widget/codeEditor/codeEditorWidget.js';
 import { EmbeddedCodeEditorWidget } from '../../../../../editor/browser/widget/codeEditor/embeddedCodeEditorWidget.js';
 import { DiffEditorWidget } from '../../../../../editor/browser/widget/diffEditor/diffEditorWidget.js';
 import { EmbeddedDiffEditorWidget } from '../../../../../editor/browser/widget/diffEditor/embeddedDiffEditorWidget.js';
-import { MarkdownRenderer } from '../../../../../editor/browser/widget/markdownRenderer/browser/markdownRenderer.js';
+import { IMarkdownRendererService } from '../../../../../platform/markdown/browser/markdownRenderer.js';
 import { IDiffEditorOptions, IEditorOptions } from '../../../../../editor/common/config/editorOptions.js';
+import { ITextModel } from '../../../../../editor/common/model.js';
 import { IResolvedTextEditorModel, ITextModelService } from '../../../../../editor/common/services/resolverService.js';
 import { peekViewResultsBackground } from '../../../../../editor/contrib/peekView/browser/peekView.js';
 import { localize } from '../../../../../nls.js';
+import { IConfigurationService } from '../../../../../platform/configuration/common/configuration.js';
 import { IInstantiationService } from '../../../../../platform/instantiation/common/instantiation.js';
 import { TerminalCapability } from '../../../../../platform/terminal/common/capabilities/capabilities.js';
 import { TerminalCapabilityStore } from '../../../../../platform/terminal/common/capabilities/terminalCapabilityStore.js';
 import { formatMessageForTerminal } from '../../../../../platform/terminal/common/terminalStrings.js';
 import { IWorkspaceContextService } from '../../../../../platform/workspace/common/workspace.js';
+import { IEditorConfiguration } from '../../../../browser/parts/editor/textEditor.js';
 import { EditorModel } from '../../../../common/editor/editorModel.js';
 import { PANEL_BACKGROUND, SIDE_BAR_BACKGROUND } from '../../../../common/theme.js';
 import { IViewDescriptorService, ViewContainerLocation } from '../../../../common/views.js';
+import { CALL_STACK_WIDGET_HEADER_HEIGHT } from '../../../debug/browser/callStackWidget.js';
 import { DetachedProcessInfo } from '../../../terminal/browser/detachedTerminal.js';
 import { IDetachedTerminalInstance, ITerminalService } from '../../../terminal/browser/terminal.js';
 import { getXtermScaledDimensions } from '../../../terminal/browser/xterm/xtermTerminal.js';
 import { TERMINAL_BACKGROUND_COLOR } from '../../../terminal/common/terminalColorRegistry.js';
-import { colorizeTestMessageInEditor } from '../testMessageColorizer.js';
-import { InspectSubject, MessageSubject, TaskSubject, TestOutputSubject } from './testResultsSubject.js';
 import { Testing } from '../../common/constants.js';
 import { MutableObservableValue } from '../../common/observableValue.js';
 import { ITaskRawOutput, ITestResult, ITestRunTaskResults, LiveTestResult, TestResultItemChangeReason } from '../../common/testResult.js';
 import { ITestMessage, TestMessageType, getMarkId } from '../../common/testTypes.js';
-import { ScrollEvent } from '../../../../../base/common/scrollable.js';
-import { CALL_STACK_WIDGET_HEADER_HEIGHT } from '../../../debug/browser/callStackWidget.js';
-import { ITextModel } from '../../../../../editor/common/model.js';
+import { colorizeTestMessageInEditor } from '../testMessageColorizer.js';
+import { InspectSubject, MessageSubject, TaskSubject, TestOutputSubject } from './testResultsSubject.js';
 
 
 class SimpleDiffEditorModel extends EditorModel {
@@ -66,7 +67,7 @@ class SimpleDiffEditorModel extends EditorModel {
 
 
 export interface IPeekOutputRenderer extends IDisposable {
-	onDidContentSizeChange?: Event<void>;
+	readonly onDidContentSizeChange?: Event<void>;
 	onScrolled?(evt: ScrollEvent): void;
 	/** Updates the displayed test. Should clear if it cannot display the test. */
 	update(subject: InspectSubject): Promise<boolean>;
@@ -110,6 +111,35 @@ const diffEditorOptions: IDiffEditorConstructionOptions = {
 	diffAlgorithm: 'advanced',
 };
 
+function applyEditorMirrorOptions<T extends IEditorOptions>(base: T, cfg: IConfigurationService, update: (options: Partial<IEditorOptions>) => void) {
+	const immutable = new Set(Object.keys(base));
+	function applyCurrent() {
+		const configuration = cfg.getValue<IEditorConfiguration>('editor');
+
+		let changed = false;
+		const patch: Partial<IEditorOptions> = {};
+		for (const [key, value] of Object.entries(configuration)) {
+			if (!immutable.has(key) && (base as Record<string, unknown>)[key] !== value) {
+				(patch as Record<string, unknown>)[key] = value;
+				changed = true;
+			}
+		}
+
+		return changed ? patch : undefined;
+	}
+
+	Object.assign(base, applyCurrent());
+
+	return cfg.onDidChangeConfiguration(e => {
+		if (e.affectsConfiguration('editor')) {
+			const patch = applyCurrent();
+			if (patch) {
+				update(patch);
+				Object.assign(base, patch);
+			}
+		}
+	});
+}
 
 export class DiffContentProvider extends Disposable implements IPeekOutputRenderer {
 	private readonly widget = this._register(new MutableDisposable<DiffEditorWidget>());
@@ -126,6 +156,7 @@ export class DiffContentProvider extends Disposable implements IPeekOutputRender
 		private readonly container: HTMLElement,
 		@IInstantiationService private readonly instantiationService: IInstantiationService,
 		@ITextModelService private readonly modelService: ITextModelService,
+		@IConfigurationService private readonly configurationService: IConfigurationService,
 	) {
 		super();
 	}
@@ -148,21 +179,32 @@ export class DiffContentProvider extends Disposable implements IPeekOutputRender
 
 		const model = this.model.value = new SimpleDiffEditorModel(original, modified);
 		if (!this.widget.value) {
-			this.widget.value = this.editor ? this.instantiationService.createInstance(
+			const options = { ...diffEditorOptions };
+			const listener = applyEditorMirrorOptions(
+				options,
+				this.configurationService,
+				u => editor.updateOptions(u)
+			);
+
+			const editor = this.widget.value = this.editor ? this.instantiationService.createInstance(
 				EmbeddedDiffEditorWidget,
 				this.container,
-				diffEditorOptions,
+				options,
 				{},
 				this.editor,
 			) : this.instantiationService.createInstance(
 				DiffEditorWidget,
 				this.container,
-				diffEditorOptions,
+				options,
 				{},
 			);
 
+			Event.once(editor.onDidDispose)(() => {
+				listener.dispose();
+			});
+
 			if (this.dimension) {
-				this.widget.value.layout(this.dimension);
+				editor.layout(this.dimension);
 			}
 		}
 
@@ -191,6 +233,7 @@ export class DiffContentProvider extends Disposable implements IPeekOutputRender
 			editor.getOriginalEditor().getContentHeight(),
 			editor.getModifiedEditor().getContentHeight()
 		);
+		editor.updateOptions({ scrollbar: { ...commonEditorOptions.scrollbar, handleMouseWheel: !hasMultipleFrames } });
 		this.helper = new ScrollHelper(hasMultipleFrames, height, dimensions.height);
 		return height;
 	}
@@ -208,14 +251,15 @@ export class DiffContentProvider extends Disposable implements IPeekOutputRender
 
 
 export class MarkdownTestMessagePeek extends Disposable implements IPeekOutputRenderer {
-	private readonly markdown = new Lazy(
-		() => this.instantiationService.createInstance(MarkdownRenderer, {}),
-	);
+
 	private readonly rendered = this._register(new DisposableStore());
 
 	private element?: HTMLElement;
 
-	constructor(private readonly container: HTMLElement, @IInstantiationService private readonly instantiationService: IInstantiationService) {
+	constructor(
+		private readonly container: HTMLElement,
+		@IMarkdownRendererService private readonly markdownRendererService: IMarkdownRendererService,
+	) {
 		super();
 		this._register(toDisposable(() => this.clear()));
 	}
@@ -232,7 +276,7 @@ export class MarkdownTestMessagePeek extends Disposable implements IPeekOutputRe
 		}
 
 
-		const rendered = this.rendered.add(this.markdown.value.render(message.message, {}));
+		const rendered = this.rendered.add(this.markdownRendererService.render(message.message, {}));
 		rendered.element.style.userSelect = 'text';
 		rendered.element.classList.add('preview-text');
 		this.container.appendChild(rendered.element);
@@ -293,6 +337,7 @@ export class PlainTextMessagePeek extends Disposable implements IPeekOutputRende
 		private readonly container: HTMLElement,
 		@IInstantiationService private readonly instantiationService: IInstantiationService,
 		@ITextModelService private readonly modelService: ITextModelService,
+		@IConfigurationService private readonly configurationService: IConfigurationService,
 	) {
 		super();
 	}
@@ -311,21 +356,32 @@ export class PlainTextMessagePeek extends Disposable implements IPeekOutputRende
 
 		const modelRef = this.model.value = await this.modelService.createModelReference(subject.messageUri);
 		if (!this.widget.value) {
-			this.widget.value = this.editor ? this.instantiationService.createInstance(
+			const options = { ...commonEditorOptions };
+			const listener = applyEditorMirrorOptions(
+				options,
+				this.configurationService,
+				u => editor.updateOptions(u)
+			);
+
+			const editor = this.widget.value = this.editor ? this.instantiationService.createInstance(
 				EmbeddedCodeEditorWidget,
 				this.container,
-				commonEditorOptions,
+				options,
 				{},
 				this.editor,
 			) : this.instantiationService.createInstance(
 				CodeEditorWidget,
 				this.container,
-				commonEditorOptions,
+				options,
 				{ isSimpleWidget: true }
 			);
 
+			Event.once(editor.onDidDispose)(() => {
+				listener.dispose();
+			});
+
 			if (this.dimension) {
-				this.widget.value.layout(this.dimension);
+				editor.layout(this.dimension);
 			}
 		}
 
@@ -355,6 +411,8 @@ export class PlainTextMessagePeek extends Disposable implements IPeekOutputRende
 		editor.layout(dimensions);
 		const height = editor.getContentHeight();
 		this.helper = new ScrollHelper(hasMultipleFrames, height, dimensions.height);
+		editor.updateOptions({ scrollbar: { ...commonEditorOptions.scrollbar, handleMouseWheel: !hasMultipleFrames } });
+
 		return height;
 	}
 }
