@@ -47,6 +47,7 @@ import { UserDataProfileService } from '../../../userDataProfile/common/userData
 import { IUserDataProfileService } from '../../../userDataProfile/common/userDataProfile.js';
 import { IBrowserWorkbenchEnvironmentService } from '../../../environment/browser/environmentService.js';
 import { ensureNoDisposablesAreLeakedInTestSuite } from '../../../../../base/test/common/utils.js';
+import { PolicyCategory } from '../../../../../base/common/policy.js';
 
 const ROOT = URI.file('tests').with({ scheme: 'vscode-tests' });
 
@@ -89,7 +90,9 @@ suite('ConfigurationEditing', () => {
 					'default': 'isSet',
 					policy: {
 						name: 'configurationEditing.service.policySetting',
+						category: PolicyCategory.Extensions,
 						minimumVersion: '1.0.0',
+						localization: { description: { key: '', value: '' } }
 					}
 				}
 			}
@@ -336,6 +339,18 @@ suite('ConfigurationEditing', () => {
 		assert.strictEqual(parsed['my.super.setting'], 'my.super.value');
 	});
 
+	test('write user standalone mcp setting - existing file', async () => {
+		const target = joinPath(environmentService.userRoamingDataHome, USER_STANDALONE_CONFIGURATIONS['mcp']);
+		await fileService.writeFile(target, VSBuffer.fromString('{ "my.super.setting": "my.super.value" }'));
+
+		await testObject.writeConfiguration(EditableConfigurationTarget.USER_LOCAL, { key: 'mcp.service.testSetting', value: 'value' });
+
+		const contents = await fileService.readFile(target);
+		const parsed = json.parse(contents.value.toString());
+		assert.strictEqual(parsed['service.testSetting'], 'value');
+		assert.strictEqual(parsed['my.super.setting'], 'my.super.value');
+	});
+
 	test('write workspace standalone setting - empty file - full JSON', async () => {
 		await testObject.writeConfiguration(EditableConfigurationTarget.WORKSPACE, { key: 'tasks', value: { 'version': '1.0.0', tasks: [{ 'taskName': 'myTask' }] } });
 
