@@ -140,15 +140,15 @@ export class ProgressService extends Disposable implements IProgressService {
 		return promise.finally(() => clearTimeout(delayHandle));
 	}
 
-	private updateWindowProgress(idx: number = 0) {
+	private updateWindowProgress(idx = 0) {
 
 		// We still have progress to show
 		if (idx < this.windowProgressStack.length) {
 			const [options, progress] = this.windowProgressStack[idx];
 
 			const progressTitle = options.title;
-			const progressMessage = progress.value && progress.value.message;
-			const progressCommand = (<IProgressWindowOptions>options).command;
+			const progressMessage = progress.value?.message;
+			const progressCommand = options.command;
 			let text: string;
 			let title: string;
 			const source = options.source && typeof options.source !== 'string' ? options.source.label : options.source;
@@ -338,6 +338,7 @@ export class ProgressService extends Disposable implements IProgressService {
 			// shows again.
 			let windowProgressDisposable: IDisposable | undefined = undefined;
 			const onVisibilityChange = (visible: boolean) => {
+
 				// Clear any previous running window progress
 				dispose(windowProgressDisposable);
 
